package com.blueworld.lassopro;

/**
 * <p>The core of the Lasso Java API v8.0.
 *
 * All the interaction between an LJAPI module and Lasso Professional is achieved 
 * by a means of invoking various methods implemented in this class. 
 * These functions can be used to perform the following tasks:<br> 
 * <ul>
 * <li>register your tags or data sources</li>
 * <li>allocate memory</li>
 * <li>return error messages</li>
 * <li>get tag or parameter information</li> 
 * <li>get client/server environment information</li>
 * <li>output text/data</li>
 * <li>read/set MIME headers</li>
 * <li>access LDML variables</li>
 * <li>interpret/execute arbitrary LDML tags</li>
 * <li>store persistent data</li>
 * <li>check if the user is an administrator</li>
 * <li>perform data source functions</li>
 * <li>safely access multiuser/multithreaded resources</li>
 * </ul>
 * <p>Most methods return the result code - either <code>NOERR</code>
 * if the function succeeds, or an error code indicating there was a problem during this method execution.
 * @see LassoDSModule LassoDSModule
 * @see LassoTagModule LassoTagModule
 * @see LassoErrors LassoErrors
 * 
 */

public class LassoCall {

	private int nativeRef;
	
	protected int NOERR = 0;

/**
	Returns <code>true</code> if a native object instance this LassoCall refers to 
	has not been destroyed yet. Always evaluates to <code>false</code> when called 
	outside the scope of the current call.
 */
	public boolean isValid()
	{
		return ( nativeRef != 0 );
	}
	
	protected LassoCall () { ; }
	
	static private final java.lang.String version = "Lasso Java API v8.0, 9/21/2004";

/**
 	Returns LJAPI version string
 	
 	@omnipilot
 */	
	static public String getVersion()
	{
		return version;
	}
		
/*********************************************************
	CONSTANT DEFINITIONS
**********************************************************/

 /**
	Register a constant value.
	
	Constants can be called just like tags, but the resulting value will only have a single instance and will have both it's type and value frozen.
	This is usually called at the same time that tags are registered.
	
	@param	name	the name for the constant
	@param	val		the value for the constant
*/
	//osError lasso_registerConstant( const char * name, lasso_type_t val );
	public native static int registerConstant( String name, LassoTypeRef val );

/**
	Register a constant value.
	
	Constants can be called just like tags, but the resulting value will only have a single instance and will have both it's type and value frozen.
	This is usually called at the same time that tags are registered

	@param	namespaceName	the namespace for the constant
	@param	name			the name for the constant
	@param	val				the value for the constant
	
	@since v8.0
*/
	//osError lasso_registerConstant2( const char * namespaceName, const char * name, lasso_type_t val );
	public native static int registerConstant2( String namespaceName, String name, LassoTypeRef val );


/*********************************************************
	MODULE REGISTRATION
**********************************************************/
/*
	All Lasso modules must implement a method named "registerLassoModule".
	Modules then call back into Lasso using the following two routines:
	@see com.blueworld.lassorpo.LassoDSModule#registerDSModule
	@see com.blueworld.lassorpo.LassoTagModule#registerTagModule
*/

	
/*********************************************************
	REQUEST PARAMS
**********************************************************/
/**
	Fetches a request-specific variable.
	
	@param	key			one of the {@link RequestParams RequestParams} constants identifying the request variable to retrieve.
	@param	outParam	the resulting data for the requested variable. Some request variables require additional input data, which is submitted in this parameter
	
	@see RequestParams RequestParams
	
	@blueworld
*/
	//osError lasso_getRequestParam( lasso_request_t token, RequestParamKeyword key, auto_lasso_value_t * result );
	public native int getRequestParam( int key, LassoValue outParam);

/**
	Fetches a request-specific variable.<br>
	
	<i>Note:</i> If requested variable requires additional input data, use {@link #getRequestParam(int,LassoValue) getRequestParam} method instead
	
	@param	key		one of the {@link RequestParams RequestParams} constants identifying the request variable to retrieve.
	@return 		valid param value, or <code>null</code> on error

	@see 	RequestParams RequestParams	
	@since 	v8.0
	
	@omnipilot
*/
	private String getRequestParam( int key )
	{
		LassoValue val = new LassoValue();
		return ( this.getRequestParam(key, val) != NOERR ) ? null : val.name();
	}
	
/**
	Fetches an internal server-specific variable.
	
	@param	key			one of the {@link LassoParams LassoParams} constants identifying the server variable to retrieve
	@param	outParam	the resulting value for the server variable.
	
	@see LassoParams 
	
	@blueworld
*/
	//osError lasso_getLassoParam( lasso_request_t token, LassoParamKeyword key, auto_lasso_value_t * result );
	public native int getLassoParam( int key, LassoValue outParam );

/**
	Fetches an internal server-specific variable.
	
	@param	key	one of the {@link LassoParams RequestParams} constants identifying the server variable to retrieve.
	@return the resulting value for the server variable, or <code>null</code> on error

	@see 	LassoParams
	@since	v8.0

 	@omnipilot
*/
	private String getLassoParam( int key )
	{
		LassoValue val = new LassoValue();
		return ( this.getLassoParam(key, val) != NOERR ) ? null : val.name();
	}
	
/**
	Returns <code>NOERR</code> if Lasso edition is LassoLite.
*/
	//osError lasso_isLassoLite( lasso_request_t token );
	public native int isLassoLite();


/*********************************************************
	ERROR/RESULT CODES
**********************************************************/
/**
 	Sets the result code that can be displayed.
 
 	@param	err	the result code
 */
	//osError lasso_setResultCode( lasso_request_t token, osError err );
	public native int setResultCode( int err );
	
/**
  	Sets the result message string
  	
  	@param	err	the result message 	
 */
	//osError lasso_setResultMessage( lasso_request_t token, const char * msg );
	public native int setResultMessage( String msg );


/*********************************************************
	TAG MODULE FUNCTIONS
**********************************************************/
/**
	Fetches the name of the tag that triggered this call. 
	For example, in the case of: <code>[my_tag: ...]</code>
	the resulting value would be <i>&quot;my_tag&quot;</i>
	
	@param	outName	The resulting tag name which will be placed in the <code>name</code> field of the {@link LassoValue LassoValue} object.
	
	@blueworld
*/
	//osError lasso_getTagName( lasso_request_t token, auto_lasso_value_t * result);
	public native int getTagName( LassoValue outName );
	
/**
	Fetches the name of the tag that triggered this call. 
	
	@return The resulting tag name, or <code>null</code> on error
	
	@since v8.0

 	@omnipilot
*/	
	private String getTagName( )
	{
		LassoValue outName = new LassoValue();
		return (NOERR != this.getTagName( outName )) ? null : outName.name();
	}
	
/**
	Fetches the number of parameters that were passed to the tag.
	
	@param	outCount	an object in which to store the result.
	
	@blueworld
*/
	//osError lasso_getTagParamCount( lasso_request_t token, int * result );
	public native int getTagParamCount( IntValue outCount );

/**
	Fetches the number of parameters that were passed to the tag.
		
	@return a positive number (<code>&gt;=0</code>) on success, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getTagParamCount( )
	{
		com.blueworld.lassopro.IntValue outCount = new com.blueworld.lassopro.IntValue ();
		return (NOERR != this.getTagParamCount( outCount )) ? -1 : outCount.intValue();
	}
	
/**
	Fetches the indicated parameter that was sent to the tag.
	
	Parameter indexes start at zero. If the parameter was specified as a keyword/value pair, 
	the keyword will be placed in the <code>name</code> field, and the value in the <code>data</code>.
	If the parameter was provided without a keyword, the <code>name</code> portion of the struct will be <code>null</code>.
	
	All parameters values will be converted to string, regardless of the original type
	
	@param	paramIndex	the zero-based index of the desired parameter
	@param	outParam	the resulting parameter data
	
	@blueworld
*/
	//osError lasso_getTagParam( lasso_request_t token, int paramIndex, auto_lasso_value_t * result );
	public native int getTagParam( int paramIndex, LassoValue outParam );

/**
	Fetches the indicated parameter that was sent to the tag.
		
	@param	paramIndex	the zero-based index of the desired parameter
	@return {@link LassoValue LassoValue} on success, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue getTagParam( int paramIndex )
	{
		LassoValue outParam = new LassoValue();
		return (NOERR != this.getTagParam( paramIndex, outParam )) ? null : outParam;
	}

/**
	Returns <code>NOERR</code> if the parameter was defined, anything else means it wasn't.
	Only parameters specified with a keyword should be searched for.
	
	@param	paramName	rhe name of the keyworded parameter to search for.
 */
	//osError lasso_tagParamIsDefined( lasso_request_t token, const char * paramName );
	public native int tagParamIsDefined( String paramName );
	
/**
	Finds and fetches a tag param by name.

	A return value of <code>NOERR</code> means the param was found.
	The parameter that was passed to the LJAPI tag call must have been specified by keyword.

	The keyword will be placed in the <code>name</code> field of the {@link LassoValue LassoValue} object, and the value - in the <code>data</code> portion. 
	If only a keyword was provided, the <code>data</code> field will be <code>null</code>.
	
	@param	paramName	the keyword of the desired parameter.
	@param	outParam	the resulting parameter value.
	
	@blueworld
*/

	//osError lasso_findTagParam( lasso_request_t token, const char * paramName, auto_lasso_value_t * result );
	public native int findTagParam( String paramName, LassoValue outParam );

/**
	Finds and fetches a tag param by name.
	
	The keyword will be placed in the <code>name</code> field of the {@link LassoValue LassoValue} object, and the value - in the <code>data</code> portion. 
	If only a keyword was provided, the <code>data</code> field will be <code>null</code>.
	
	@param	paramName	the keyword of the desired parameter.
	@return				valid {@link LassoValue LassoValue} instance, or <code>null</code> on error	

	@since	v8.0
	
 	@omnipilot
*/
	private LassoValue findTagParam( String paramName )
	{
		LassoValue outParam = new LassoValue();
		return (NOERR != this.findTagParam( paramName, outParam )) ? null : outParam;
	}
	
/**
	Fetches the encoding method indicated for this tag.
	
	@deprecated
*/
	// osError lasso_getTagEncoding( lasso_request_t token, encodingMethod * method );
	public native int getTagEncoding( IntValue outMethod ); // FIX!

/**
	Output some data onto the page.
	
	@deprecated
*/
	// osError lasso_outputTagData( lasso_request_t token, const char * data );
	public native int outputTagData( String data );

/**
	Output some data onto the page. Suitable for returning binary data.
*/
	// osError lasso_outputTagData( lasso_request_t token, const char * data );
	public native int outputTagData( byte[] data );

/**
	Output some data onto the page.
	Alias for {@link #returnTagValueBytes returnTagValueBytes}.

*/
	public int outputTagBytes( byte[] data )
	{ 
		return outputTagData(data); 
	}
	
	//lasso_outputTagDataF takes the same formatting flags as printf

/**
	Allows access to the result (HTTP) header.
	The data will be a copy of the current HTTP header data.
	
	@param	outHeader	on return, contains the header data in the <code>name</code> field
	
	@blueworld
*/
	//osError lasso_getResultHeader( lasso_request_t token, auto_lasso_value_t * result );
	public native int getResultHeader( LassoValue outHeader );

/**
	Allows access to the result (HTTP) header.
		
	@return the valid header data, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private String getResultHeader( )
	{
		LassoValue header = new LassoValue();
		return (NOERR != this.getResultHeader( header )) ? null : header.name();
	}
	
/**
	Sets the result (HTTP) header.
	The data will be set as-is, so make sure it is properly formatted. 
	The data should not include the final trailing CRLF pair.
	
	@param	header	the header data to use
*/
	//osError lasso_setResultHeader( lasso_request_t token, const char * header );
	public native int setResultHeader( String header );
		
/**
	Appends the supplied data to the header.
	The data will be set as-is, so make sure it is properly formatted. 
	The data should not include the final trailing CRLF pair.
	
	@param	header	the header data to use
*/
	//osError lasso_addResultHeader( lasso_request_t token, const char * data );
	public native int addResultHeader( String header );

/**
	Retrieves a cookie value set by the client.
	
	@param	named	the name of the cookie to look for
	@param	outValue	the resulting cookie value which will be placed in the <code>name</code> portion of the struct.
	
	@blueworld
*/
	
	//osError lasso_getCookieValue( lasso_request_t token, const char * named, auto_lasso_value_t * value );
	public native int getCookieValue( String named, LassoValue outValue );
	
/**
	Retrieves a cookie value set by the client.
		
	@param	named	the name of the cookie to look for
	@return 		the resulting cookie value, or <code>null</code> if cookie wasn't found
	
	@since v8.0
	
 	@omnipilot
*/
	// NOTE: check for valid ret codes with KJ (lcapi err vs. cookie not found).
	public String getCookieValue( String named )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getCookieValue( named, outValue )) ? null : outValue.name();
	}

/*
	Retrieves the number of variables with the specified name.
	
	@param	named	name of a variable
	@param	outCount	count of variables with the specified name
*/
/**
	@deprecated
*/
	//osError lasso_getVariableCount( lasso_request_t token, const char * named, int * count );
	public native int getVariableCount( String named, IntValue outCount );
/*
	Retrieves the specified variable value
	@param	named		name of a variable
	@param	index		indexes are zero-based
	@param	outValue	value of the named variable
*/
/**
	@deprecated
*/
	//osError lasso_getVariable( lasso_request_t token, const char * named, int index, auto_lasso_value_t * value );
	public native int getVariable( String named, int index, LassoValue outValue );
	
/*
	Sets the specified variable.
*/
/**
	@deprecated
*/
	//osError lasso_setVariable( lasso_request_t token, const char * named, const char * value, int index );
	public native int setVariable( String named, String value, int index );
	
/*
	Removes the specified variable.
*/
/**
	@deprecated
*/
	//osError lasso_removeVariable( lasso_request_t token, const char * named, int index );
	public native int removeVariable( String named, int index );

/**
	!FIX!
	FOR CONTAINER TAGS ONLY
	... describe this and work out the logistics...
*/
/* osError lasso_serviceTagContents( lasso_request_t token ); */


/*********************************************************
	DATASOURCE MODULE FUNCTIONS
**********************************************************/

/**
	Adds a datasource result value.
	
	Datasource actions which require returning multiple values can use this to add those values.
	For example, this call can be used to add the name of a datasource that the module services 
	or the names of the tables in a particular datasource.

	<code>getInputColumn</code> & <code>findInputColumn</code> can be used by tags to get input form params.
	
	@param	data	the data to add. <i>(TBD: Data must be in ISO8859-1 encoding.)</i>
*/
	//osError lasso_addDataSourceResult( lasso_request_t token, const char * data );
	public native int addDataSourceResult( String data );
	
/**
	Gets the currently specified database name and associated data.
	
	@param	outName				on return, contains the database name stored in the <code>data</code> field.
	@param	outUseHostDefault	the setting which specifies whether the hosts username/password combo should be used in the absence of a database specific combo
	@param	outUsernamePassword	user name and password to be used for logging in to the datasource host.
*/
	//osError lasso_getDataSourceName( lasso_request_t token, auto_lasso_value_t * t, bool * useHostDefault, auto_lasso_value_t * usernamepassword );
	public native int getDataSourceName( LassoValue outName, IntValue outUseHostDefault, LassoValue outUsernamePassword ); // inLassoValue !FIX! why bool * ?

/**
	Gets the currently specified table name.
	@param	outName	on return, contains the table name stored in the <code>name</code> field.
	
	@blueworld
*/
	//osError lasso_getTableName( lasso_request_t token, auto_lasso_value_t * t );
	public native int getTableName( LassoValue outName );

/**
	Gets the currently specified table name.
		
	@return the table name, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private String getTableName( )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getTableName( outValue )) ? null : outValue.name();
	}
	
/**
	Gets the currently specified schema name.
	
	@param	outName		on return, contains the schema name stored in the <code>name</code> field.
	
	@blueworld
*/
	//osError lasso_getSchemaName( lasso_request_t token, auto_lasso_value_t * t );
	public native int getSchemaName( LassoValue outName );

/**
	Gets the currently specified schema name.
		
	@return the schema name, or <code>null</code> on error

	@since v8.0
	
 	@omnipilot
*/
	private String getSchemaName( )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getSchemaName( outValue )) ? null : outValue.name();
	}	

/**
	Returns the host that maintains the current database.
	
	@param	outHost				the host address and port number strings are returned in the <code>name</code> and <code>data</code> fields, respectively
	@param	outUsernamePassword	host default username and password, if any
*/
	//osError lasso_getDataHost( lasso_request_t token, auto_lasso_value_t * host, auto_lasso_value_t * usernamepassword );
	public native int getDataHost( LassoValue outHost, LassoValue outUsernamePassword );
	
/**
	Returns the host that maintains the current database. Similar to the previous method, but also returns default host schema.
*/
	//osError lasso_getDataHost( lasso_request_t token, auto_lasso_value_t * host, auto_lasso_value_t * schema, auto_lasso_value_t * usernamepassword );
	public native int getDataHost( LassoValue outHost, LassoValue outDefaultSchema, LassoValue outUsernamePassword );
	
/**
	Returns the host id (for internal use only).
*/
	public native int getDataHostID( IntValue outID );
	
/**
	The number of rows that should be skipped in the found set.
	See <code>-skiprecords</code> LDML parameter reference for more info.
	
	@blueworld
*/

	//osError lasso_getSkipRows( lasso_request_t token, int * recs );
	public native int getSkipRows( IntValue outRows );
	
/**
	The number of rows that should be skipped in the found set.
		
	@return the skip row count, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getSkipRows( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getSkipRows( outValue )) ? 0 : outValue.intValue();
	}
	
/**
	The maximum number of rows in the found set to return.
	See <code>-skiprecords</code> LDML parameter reference for more info.
	
	@blueworld
*/
	//osError lasso_getMaxRows( lasso_request_t token, int * recs );
	public native int getMaxRows( IntValue outRows );
	
/**
	The number of rows that should be skipped in the found set.
		
	@return the max row count, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getMaxRows( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getMaxRows( outValue )) ? -1 : outValue.intValue();
	}
	
/**
	Gets the currently specified record id (FileMaker-specific).	
	See <code>-keyvalue</code> LDML parameter reference for more info.
	
	@param	outId	the resulting numeric <code>-keyvalue</code> param value
	
	@blueworld
*/
	//osError lasso_getRowID( lasso_request_t token, int * id );
	public native int getRowID( IntValue outId );

/**
	Gets the currently specified record id (FileMaker-specific).
		
	@return the resulting numeric <code>-keyvalue</code> value
	
	@since v8.0
	
 	@omnipilot
*/
	private int getRowID( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getRowID( outValue )) ? 0 : outValue.intValue();
	}
	
/**
	Sets the currently specified record id (FileMaker-specific).
	
	@param	id	the numeric row id.
*/
	//osError lasso_setRowID( lasso_request_t token, int id );
	public native int setRowID( int id );
	
/**
	Gets the name and the value of the currently specified primary key column.
	
	@param	outColumn	on return, contains the resulting key name and value.
	
	@blueworld
*/
	//osError lasso_getPrimaryKeyColumn( lasso_request_t token, auto_lasso_value_t * v );
	public native int getPrimaryKeyColumn( LassoValue outColumn );

/**
	Gets the name and the value of the currently specified primary key column.
		
	@return the resulting key name and value, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue getPrimaryKeyColumn( )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getPrimaryKeyColumn( outValue )) ? null : outValue;
	}
	
/**
	Gets the name and the value of the indexed primary key column.
	
	@param	index		the zero-based column index
	@param	outColumn	on return, contains the resulting key name and value
	
	@blueworld
*/
	//osError lasso_getPrimaryKeyColumn( lasso_request_t token, int, index, auto_lasso_value_t * v );
	public native int getPrimaryKeyColumn2( int index, LassoValue outColumn );

/**
	Gets the name and the value of the indexed primary key column.
		
	@return the resulting key name and value, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue getPrimaryKeyColumn( int index )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getPrimaryKeyColumn2( index, outValue )) ? null : outValue;
	}
	
/**
	Gets the number of input columns for this database action.
	
	@blueworld
*/
	//osError lasso_getInputColumnCount( lasso_request_t token, int * count );
	public native int getInputColumnCount( IntValue outCount );

/**
	Gets the number of input columns for this database action.
		
	@return the input column count, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getInputColumnCount( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getInputColumnCount( outValue )) ? -1 : outValue.intValue();
	}
	
/**
	Gets the number of sort columns for this database action.
	
	@blueworld
*/
	//osError lasso_getSortColumnCount( lasso_request_t token, int * count );
	public native int getSortColumnCount( IntValue outCount );

/**
	Gets the number of sort columns for this database action.
		
	@return the input column count, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getSortColumnCount( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getSortColumnCount( outValue )) ? -1 : outValue.intValue();
	}
	
/**
	Gets an individual input column by index.
	
	@param	index	the zero-based index of the column to retrieve.
	
	@blueworld
*/
	//osError lasso_getInputColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
	public native int getInputColumn( int index, LassoValue outColumn );

/**
	Gets an individual input column by index.
		
	@param	index	the zero-based index of the column to retrieve
	@return 		the resulting key name and value, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue getInputColumn( int index )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getInputColumn( index, outValue )) ? null : outValue;
	}
	
/**
	Gets an individual sort column by index.
	
	@param	index	the zero-based index of the sort column to retrieve
	@return 		the resulting column name and value, or <code>null</code> on error
	
	@blueworld
*/
	//osError lasso_getSortColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
	public native int getSortColumn( int index, LassoValue outColumn );

/**
	Gets an individual sort column by index.
		
	@param	index	the zero-based index of the column to retrieve.
	@return 		the resulting column name and value, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue getSortColumn( int index )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getSortColumn( index, outValue )) ? null : outValue;
	}
	
/**
	Gets an individual input column by name.
	
	@param	named	the name of the input column to retrieve
	@param	value	the resulting value
	
	@blueworld
*/
	//osError lasso_findInputColumn( lasso_request_t token, const char * name, auto_lasso_value_t * value );
	public native int findInputColumn( String named, LassoValue outColumn );

/**
	Gets an individual input column by name.
		
	@param	index	the zero-based index of the column to retrieve
	@return 		the resulting column name and value, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue findInputColumn( String named )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.findInputColumn( named, outValue )) ? null : outValue;
	}
	
/**
	Gets the logical operator for this database action.
	
	@param	outOp	on return, one of the {@link LassoOperators LassoOperators} constants
	
	@blueworld
*/
	//osError lasso_getLogicalOp( lasso_request_t token, DescType * op );
	public native int getLogicalOp( IntValue outOp );

/**
	Gets the logical operator for this database action.
		
	@return the resulting value, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getLogicalOp( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getSortColumnCount( outValue )) ? -1 : outValue.intValue();
	}	
	
/**
	Gets the number of return columns for this action.
	
	@param	outCount	on return, contains the resulting number of return columns
	
	@blueworld
*/
	//osError lasso_getReturnColumnCount( lasso_request_t token, int * count );
	public native int getReturnColumnCount( IntValue outCount );

/**
	Gets the number of return columns for this action.
		
	@return the resulting column count, or <code>-1</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private int getReturnColumnCount( )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (NOERR != this.getReturnColumnCount( outValue )) ? -1 : outValue.intValue();
	}
	
/**
	Gets an individual return column by index.
	
	@param	index		the zero-based index of the return column to retrieve
	@param	outColumn	on return, contains the resulting column name and value
	
	@blueworld
*/
	//osError lasso_getReturnColumn( lasso_request_t token, int num, auto_lasso_value_t * v );
	public native int getReturnColumn( int index, LassoValue outColumn );

/**
	Gets an individual return column by index.
		
	@param	index	the zero-based index of the column to retrieve
	@return 		the resulting column name and value, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoValue getReturnColumn( int index )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getReturnColumn( index, outValue )) ? null : outValue;
	}
/**
	Adds information about a particular column.<br>
	
	Column information should be added in the order in which the columns occur in the database.
	Column information should be added no matter what the action. For the show action, only column information is added.
	
	@param	name		the column name
	@param	nullOK		does the column type allows null values?
	@param	type		column type: TYPE_CHAR, TYPE_INT, TYPE_BOOLEAN, TYPE_DECIMAL, TYPE_DATETIME or TYPE_BLOB
	@param	protection	is column value read only? Should be either PROTECTION_NONE or PROTECTION_READ_ONLY
	
	@see LassoValue
	@see LassoDSModule
*/
	//osError lasso_addColumnInfo( lasso_request_t token, const char * name, int nullOK, DescType type, DescType protection );
	public native int addColumnInfo( String name, int nullOK, int type, int protection );

/**
	Adds information, including valuelists, about a particular column.<br>
	
	Column information should be added in the order in which the columns occur in the database.
	Column information should be added no matter what the action is. For the show action, only column information is added.

	@param	name		the column name
	@param	nullOK		is a null value for this column ok?
	@param	type		one of the data types defined in {@link LassoDSModule LassoDSModule}
	@param	protection	the code for the column's protection: either PROTECTION_NONE or PROTECTION_READ_ONLY
	@param	valueList	an array of strings for the column's valuelist.
	
	@since v8.0
	
	@omnipilot
*/	
	//osError lasso_addColumnInfo2( lasso_request_t token, const char * name, int nullOK, LP_TypeDesc type, LP_TypeDesc protection, const char ** valueList, int countValueList );
	public native int addColumnInfo2( String name, int nullOK, int type, int protection, String[] valueList );

/**
	Adds all the column data for the next result row.
	
	@param	columns	an array of column values
*/
	//osError lasso_addResultRow( lasso_request_t token, const char ** columns, unsigned int * dataSizes, int numColumns );
	public native int addResultRow( String[] columns );
	
/**
	Another way to add the column binary data for the next result row.
	<pre>
	String data[] = { "apples", "oranges", "pears", "cherries" };
	
	int count = data.length;
	byte cols[][] = new byte[count][];
	
	for ( i = 0; i < count; i++ )
	    cols[i] = data[i].getBytes();
		
	lasso.addResultRow( cols );
	</pre>
*/
	public native int addResultRow( byte[][] columns );

/**
	Sets the number of rows found in the query.
 	This will not always be the same value as the number of rows added with the <code>addResultRow</code> call.
	as the skip recs and max recs parameters effect that number.
*/
	//osError lasso_setNumRowsFound( lasso_request_t token, int num );
	public native int setNumRowsFound( int numRows );

/** 
	Returns the name the current datasource module was registered with.
	
	@param	outName	on return, contains the datasource module name in the <code>name</code> field.
	
	@blueworld
*/
	//osError lasso_getDataSourceModuleName( lasso_request_t token, auto_lasso_value_t * val );
	public native int getDataSourceModuleName( LassoValue outName );

/**
	Returns the name the current datasource module was registered with.
		
	@return the datasource module name, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private String getDataSourceModuleName( )
	{
		LassoValue outValue = new LassoValue();
		return (NOERR != this.getDataSourceModuleName( outValue )) ? null : outValue.name();
	}

/** 
	Returns the current datasource connection object, or <code>null</code> if no connection has been set yet.

	@since v7.0
*/
	//osError lasso_getDSConnection(lasso_request_t token, lasso_dsconnection_t * conn);
	public native Object getDSConnection();
	
/** 
	Called to set the current connection object for the datasource.
	May recurse to deliver the {@link LassoDSModule#ACTION_CLOSE ACTION_CLOSE} message if there is already a valid db connection.
	Once set, the connection could be retrieved from a nested LDML "inline" call.
		
	@param	inConnection	an object representing valid datasource connection.
	
	@since v7.0
*/
	//osError lasso_setDSConnection(lasso_request_t token, lasso_dsconnection_t conn);
	public native int setDSConnection( Object inConnection );

/**
	Called to set the statement for the current action.
	Datasources must call this to support the <code>action_statement</code> LDML tag.

	@param	statement	the action statement value
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_setActionStatement(lasso_request_t token, const char * stat);
	public native int setActionStatement( String statement );

/**
	Used to check for the <code>-statementonly</code> LDML inline param.

	@return <code>NOERR</code> if <code>-statementonly</code> keyword is present
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_getIsStatementOnly(lasso_request_t token, bool * out);
	public native int getIsStatementOnly( );
	
	
/*********************************************************
	SEMAPHORE FUNCTIONS
**********************************************************/

/**
	Acquires a previously created semaphore.
	
	Semaphores are a means by which access to shared resources can be controlled amongst several threads of execution.
	Each semephore must have a unique name. Always release semaphores as soon as possible.

*/
	//osError lasso_acquireSem( lasso_request_t token, const char * name );
	public native int acquireSem( String named );

/**
	Creates a named semaphore object.
	
	@see #acquireSem acquireSem
*/
	//osError lasso_createSem( lasso_request_t token, const char * name );
	public native int createSem( String named );
	
/**
	Destroys existing semaphore.
	
	@see #acquireSem acquireSem
*/
	//osError lasso_destroySem( lasso_request_t token, const char * name );
	public native int destroySem( String named );

/**
	Releases previously aquired semaphore.
	
	@see #acquireSem acquireSem
*/
	//osError lasso_releaseSem( lasso_request_t token, const char * name );
	public native int releaseSem( String named );


/*********************************************************
	FORMATTING FUNCTIONS
**********************************************************/

/** 
	Formats the supplied buffer and puts the resulting data in the <code>data</code> field of the <code>LassoValue</code>.
	
	The buffer should consist of plain text and bracketed Lasso tags.
	
	@param	buffer	the buffer holding the script data
	@param	outResult	on return, contains the resulting value
*/
	//osError lasso_formatBuffer( lasso_request_t token, const char * buffer, auto_lasso_value_t * output );
	public native int formatBuffer( String buffer, LassoValue outResult );
	
/**
	Works just like the include tag.
*/
	//osError lasso_includeRaw( lasso_request_t token, const char * path, auto_lasso_value_t * out );
	public native int includeRaw( String filePath, LassoValue out );

/*********************************************************
	STORAGE FUNCTIONS
**********************************************************/

/*
	These calls can be used to have Lasso store arbitrary persistent data. This is an excellent way to store preferences.
 */
 
/**
   Returns <code>NOERR</code> if the data, specified by key, exists.
   The length of the stored data can be returned in the length parameter.
   This parameter may be <code>null</code> if the length is not required.
 */
//osError lasso_storeHasData( lasso_request_t token, const char * key, unsigned int * length );
public native int storeHasData( String key, IntValue outLength );

/**
  Adds the data to Lasso's storage.
  <code>key</code> is the unique identifier for the data.
 */
//osError lasso_storePutData( lasso_request_t token, const char * key, const void * data, unsigned int length );
public native int storePutData( String key, String data );

/**
  Adds the data to Lasso's storage.
  <code>key</code> is the unique identifier for the data.
 */
public native int storePutData( String key, byte[] data );

/**
  Fetches data that has been stored.

  The data will be returned in the <code>data</code> field of the <code>LassoValue</code> object.
  
  @param	key	the unique identifier for the data.
 */
//osError lasso_storeGetData( lasso_request_t token, const char * key, auto_lasso_value_t * value, unsigned int * length );
public native int storeGetData( String key, LassoValue outValue );


/*********************************************************
	SECURITY FUNCTIONS
**********************************************************/

/**
	Checks the current request for administrator privileges.
	
	Returns <code>NOERR</code> is the current user has administrator priveleges.
	This is useful for doing administrative activities or file accesses which only the admin should be able to do.
 */
	//osError lasso_isAdministrator( lasso_request_t token );
	public native int isAdministrator( );

/**
	Checks the current request for the lowest privilege level.
	
	Returns <code>NOERR</code> if the current user is the anonymous user.
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_isAnonymous( lasso_request_t token );
	public native int isAnonymous( );

/**
	Switches the current request to the given user.
	
	A proper password must be provided. The switch will persist until the request is finished or is switches again.
	Pass "anonymous" with no password to switch to the lowest privileges.
	@since v7.0
*/
	//osError lasso_switchToUser( lasso_request_t token, const char * username, const char * password );
	public native int switchToUser( String username, String password );
	
/**
	Tests for file access permissions.
	
	Returns <code>NOERR</code> is the given operation is permitted by Lasso security.

	@param	secureOperation		one of the constants defined in {@link LassoSecureOperations LassoSecureOperations} class
	@param	internalPathname	full path to the file using the <b>internal pathname</b> format
 */
	//osError lasso_operationAllowed( lasso_request_t token, secure_operation_t op, void * data );
	public native int operationAllowed( int secureOperation, String internalPathname );


/*********************************************************
	PATHNAME FUNCTIONS
*********************************************************/

/**
	Converts an internal pathname into a platform-specific pathname.
	
	@param	internalPath	the internal pathname.
	@return the resulting platform specific pathname

 */
	//void lasso_getPlatformSpecificPath( const char* internalPath, osPathname platformPath );
	public static native String getPlatformSpecificPath( String internalPath );

/**
	Converts a platform-specific pathname into an internal pathname.
	
	@param	platformPath	the platform-specific path name
	@return 				the resulting internal pathname
 */
	//void lasso_getInternalPath( const char* platformPath, osPathname internalPath );
	public static native String getInternalPath( String platformPath );

/**
	Converts a relative path into a from-the-server-root path.
	If the path is already from the root it won't be changed.
	
	@param	relativePath	the relative path
	@return 				the resulting fully-qualified path
 */
	//void lasso_fullyQualifyPath( lasso_request_t token, const char * relativePath, osPathname fullyQualified );
	public native String fullyQualifyPath( String relativePath );

/**
	Converts a partial path into a full, from the file system root, internal path.
	
	@param	partialPath	the path to resolve
	@return 			the resulting full path
 */
	//void lasso_resolvePath( lasso_request_t token, const char * inFromRootPath, osPathname outFullPath );
	public native String resolvePath( String partialPath );

/**
	Converts a partial path into a full, from the file system root, internal path which is guaranteed to be within the web root.

	@param	partialPath	the path to resolve. Must name an item within the web-root or a blank string is returned
	@return 			the resulting full path	
	
	@since v8.0

	@omnipilot
*/
	//void lasso_resolveIncludePath( lasso_request_t token, const char * inPath, osPathname outFullPath );
	public native String resolveIncludePath( String partialPath );

/**
	Converts an internal path into a full platform specific path.
	
	Takes a path, such as one which may have been passed to any Lasso tag, and converts it into a full platform specific path name.

	@param	relativeOrFullInternalPath	The input path to convert
	@return 							the resulting converted path
	
	@since v8.0

	#omnipilot	
*/
	//void lasso_internalToFullPlatformPath( lasso_request_t token, const char * relativeOrFullInternalPath, osPathname fullPlatformPath );
	public native String internalToFullPlatformPath( String relativeOrFullInternalPath );

/**
	Tests the given internal path to see if it is a full path from the filesystem root.

	@param	path	the path to test
	@return <code>true</code> if the given internal path is a full path from the filesystem root
	
	@since v8.0
	
	@omnipilot
*/
	//bool lasso_isFullInternalPath(const char * path);
	public native boolean isFullInternalPath( String path );
	
/**
	Toggles how Lasso resolves file paths.
	
	Toggles whether Lasso will resolve the entire given file path (<code>/blah/file.lasso</code>) through the webserver (the default behavior: <code>true</code>)
	OR 
	whether Lasso will resolve "/" through the server and then prepend that to the given file path (<code>false</code>).

	The default behavior can conflict with some usages of mod_rewrite.
	As of LP7, this setting is global.

	@param	to	the toggle
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_setAutoResolveFullPaths(lasso_request_t token, bool to);
	public native int setAutoResolveFullPaths( boolean to );

/**
	Retreives Lasso's current file resolution mode.

	@return <code>true</code> if Lasso will resolve the given file path through the webserver

	@see #setAutoResolveFullPaths setAutoResolveFullPaths	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_getAutoResolveFullPaths(lasso_request_t token, bool * get);
	public native boolean getAutoResolveFullPaths( );

/*********************************************************
 *														 *
 *	LCAPI v6.0 FUNCTIONS								 *
 *														 *
 *********************************************************/

/*
This type is used by the developer as an opaque handle to an internally allocated Variant. 
Variants allocated in this manner are garbage collected by the API at the end of the current LCAPI function call. 
This means that lasso_type_t's are not usable in across subsequent LCAPI invocations. 
The method of garbage collection will be the same as is currently used for auto_lasso_value_t's 
in that the current CallState structure retains a list of all Variants allocated 
and destroys each of them after the return of the current LCAPI call.
*/

/*********************************************************
	ALLOCATING AND FREEING TYPE INSTANCES
**********************************************************/
/**
	Allocates a new type instance.
	The name of the type is signified by the first parameter. 
	If a type initializer is found for the given name, it will be executed.
	A type allocated in this manner will be added to the garbage list for later destruction.
	
	@param	typeName	the name of the type to instantiate
	@param	params	an array of parameters to pass to the type initializer. Could be <code>null</code>
	@param	outType	a storage for the newly instantiated type
	
	@blueworld
*/
	//osError lasso_typeAlloc ( lasso_request_t token, const char * typeName, int paramCount, lasso_type_t * params, lasso_type_t * outType );
	public native int typeAlloc ( String typeName, LassoTypeRef[] params, LassoTypeRef outType );

/**
	<code>typeAlloc</code> will attempt to allocate a new type instance. 
	
	@param	typeName	the name of the type to instantiate
	@param	params	an array of parameters to pass to the type initializer. Could be <code>null</code>
	@return The resulting type, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/	
	private LassoTypeRef typeAlloc ( String typeName, LassoTypeRef[] params )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAlloc ( typeName, params, outType ) != NOERR) ? null : outType;
	}

/**
	Allocates a custom type within a type initializer.
	
	This function is used within module methods that were registered as being a type initializer (FLAG_INITIALIZER). 
	It initializes a blank custom type and sets the type's name to the provided value. 
	The new type does not yet have a lineage and has no members added to it. 
	New data or tag members should be added using {@link #typeAddMember typeAddMember}. 
	The new type should be the return value of the tag call, set via {@link #returnTagValue returnTagValue}.
	Any inherited members will be added to the type after the LJAPI call returns.
	
	<p><b>Warning:</b> Do not call this unless you are in a type initializer. 
	If you are not in a type initializer, the result will be a type that will never be fully initialized.
	
	@param	outCustom	the resulting newly allocated custom type
	@param	typeName	the new type's name.
	
	@blueworld
*/
	//osError lasso_typeAllocCustom( lasso_request_t token, lasso_type_t * outCustom, const char * name );
	public native int typeAllocCustom( LassoTypeRef outCustom, String typeName );

/**
	Allocates a custom type within a type initializer.
	
	@param	typeName	the new type's name
	@return The resulting type, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/	
	private LassoTypeRef typeAllocCustom ( String typeName )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocCustom ( outType, typeName ) != NOERR) ? null : outType;
	}

/**
	Allocate a new type instance based on the given type instance.
	
	The given type's tag members will be referenced in the new type.
	No data members are added except for the typename member.

	@param	inProto	the original type instance whose tag members will be referenced
	@param	outType	the resulting new type instance
	
	@since v7.0
	
	@blueworld
*/
	//osError lasso_typeAllocFromProto( lasso_request_t token, lasso_type_t proto, lasso_type_t * out );
	public native int typeAllocFromProto( LassoTypeRef inProto, LassoTypeRef outType );

/**
	Allocate a new type instance based on the given type instance.
		
	The given type's tag members will be referenced in the new type.
	No data members are added except for the typename member.

	@param	inProto	the original type instance whose tag members will be referenced
	@return the resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocFromProto( LassoTypeRef inProto )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocFromProto ( inProto, outType ) != NOERR) ? null : outType;
	}
	
/**
	Allocate a new type with the given name. 
	<p />
	The type does not have to have been registered as a type initializer or registered at all.
	The new type will have no tag or data members, but those may be added using the appropriate LJAPI call at any time.
	If no parent type is provided (a <code>null</code> or empty string is passed in), type &quot;null&quot; will be the default.
	If a parent type is provided, it must have been a validly registered type initializer.
	<i>OnCreate</i> member tag will be called for the parent and any of its parents, but will not be called for the new type instance itself.
	
	@param	inName				the name for the new type instance
	@param	inParentTypeName	the name of the parent type
	@param	outType				the new type instance, or <code>null</code> on error
	
	@since v7.0
	
	@blueworld
*/
	//osError lasso_typeAllocOneOff( lasso_request_t token, const char * name, const char * parentTypeName, lasso_type_t * out );
	public native int typeAllocOneOff( String inName, String inParentTypeName, LassoTypeRef outType );

/**
	Allocate a new type with the given name. 
		<p />
	The type does not have to have been registered as a type initializer or registered *at all*.
	The new type will have no tag or data members, but those may be added using the appropriate LJAPI call at any time.
	If no parent type is provided (a <code>null</code> or empty string is passed in), type &quot;null&quot; will be the default.
	If a parent type is provided, it must have been a validly registered type initializer.
	<i>OnCreate</i> member tag will be called for the parent and any of its parents, but will not be called for the new type instance itself.
	
	@param	name			the name for the new type instance
	@param	parentTypeName	the name of the parent type
	@return 				the new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocOneOff( String name, String parentTypeName )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocOneOff ( name, parentTypeName, outType ) != NOERR) ? null : outType;
	}
	
	
/*********************************************************
	Convenience Functions For Built-in Types
**********************************************************/

/**
	Add 1 to an instance's reference count.
	
	Type instances each have a reference count.
	Types created or otherwise referenced or received during an LJAPI call can be assumed to have a reference count of at least 1.
	If client code wishes to utilize any type instance beyond the end of the LJAPI call, 
	it should add 1 to the instance's reference count. This is called "referring to" an instance.
	When client code has finished utilizing any "referred-to" instance, 
	it should subtract 1 from the instance's reference count using {@link #typeFree(LassoTypeRef) typeFree}.
	When a type instance's reference count reaches zero, the instance is destroyed.
	
	@since v8.0
*/
	//osError lasso_typeRefer( lasso_request_t token, lasso_type_t toRefer );
	public native int typeRefer( LassoTypeRef toRefer );
	
// synonymous with lasso_typeRefer. here for compatability
//osError lasso_typeDetach( lasso_request_t token, lasso_type_t toDetach );

	
/**
 	Subtract 1 from an instance's reference count.

	Once the instance's reference count reaches zero, it is destroyed and the <code>LassoTypeRef</code> is no longer valid.
	The <code>LassoCall</code> token may be <code>null</code> if it is not available for any reason.
	<b>Warning:</b> ONLY call this if you have previously called {@link #typeRefer typeRefer}
	using the same type instance or the result may be a double free.
	
	@param	inType	the type instance which should be unreferred to
*/
	//osError lasso_typeFree ( lasso_request_t token, lasso_type_t inType);
	public native int typeFree ( LassoTypeRef inType );

/**
	Remove the LJAPI call's reference to a type instance.

	The LJAPI garbage collection system usually holds a reference to any used type instances.
	Calling this will remove that reference, which may result in the instance being destroyed at that point.
	Usage of a type pointer after calling this is not permitted unless the type has been "referred" to previously using {@link #typeRefer typeRefer}.
	
	@param	inType	the type instance to operate on
*/
	//osError lasso_typeCollect( lasso_request_t token, lasso_type_t inType);
	 public native int typeCollect( LassoTypeRef inType );

/**
	Allocating Built-in Type Instances.
	
	The following functions allocate instances of specific built-in types. 
	The <code>LassoCall</code> token may be null. If it is null, the allocated types 
	are created as "detached" and must be manually freed using {@link #typeFree typeFree}
*/


/**
	Allocates a new instance of type <i>null</i>.
	
	@param	outNull	The resulting new type instance.
	
	@blueworld
*/
	//osError lasso_typeAllocNull ( lasso_request_t token, lasso_type_t * outNull );
	public native int typeAllocNull ( LassoTypeRef outNull);

/**
	Allocates a new instance of type <i>null</i>.
	
	@return the new Lasso null type instance
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocNull ( )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return this.typeAllocNull( outType ) != NOERR ? null : outType;
	}
	
/**
	Allocates a new instance of type <i>string</i>.

	@param	outString	the resulting new type instance
	@param	value	The string data from which to copy for the new string instance
	
	@blueworld
*/
	//osError lasso_typeAllocString ( lasso_request_t token, lasso_type_t * outString, const char * value, int length );
	public native int typeAllocString ( LassoTypeRef outString, String value);

/**
	Allocates a new instance of type <i>string</i>.

	@param	value	the string data from which to copy for the new Lasso string instance
	@return the resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocString ( String value )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return this.typeAllocString( outType, value ) != NOERR ? null : outType;	
	}

/**
	Allocates a new instance of type <i>string</i>.
	
	@since v7.0
*/
	public native int typeAllocString ( LassoTypeRef outString, byte[] value);

/**
	Allocates a new instance of type <i>string</i> from a Java string using specified encoding.
	
	@param	outString	the resulting new type instance
	@param	value	the source data from which to copy for the new string instance
	@param	conv	The encoding of the source data

	@since v7.0
*/
	//osError lasso_typeAllocStringConv ( lasso_request_t token, lasso_type_t * outString, const char * value, int length, const char * conv );
	public native int typeAllocStringConv ( LassoTypeRef outString, String value, String conv);

/**
	Allocates a new instance of type <i>integer</i>.
	
	@param	outInteger	the resulting new type instance
	@param	value	the integer value which the new type instance will hold
	
	@blueworld
*/
	//osError lasso_typeAllocInteger ( lasso_request_t token, lasso_type_t * outInteger, osInt64 value );
	public native int typeAllocInteger ( LassoTypeRef outInteger, long value); // may require cast: (long)int

/**
	Allocates a new instance of type <i>integer</i>.
	
	@param	value	the integer value which the new type instance will hold
	@return the resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocInteger ( long value )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return this.typeAllocInteger( outType, value ) != NOERR ? null : outType;
	}
	
/**
	Allocates a new instance of type <i>decimal</i>.
	
	@param	outDecimal	the resulting new type instance
	@param	value	the double value which the new type instance will hold
	
	@blueworld
*/
	//osError lasso_typeAllocDecimal ( lasso_request_t token, lasso_type_t * outDecimal, double value );
	public native int typeAllocDecimal ( LassoTypeRef outDecimal, double value );

/**
	Allocates a new instance of type <i>decimal</i>.
	
	@param	value	the double value which the new type instance will hold
	@return the resulting new type instance, or <code>null</code> on error
		
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef typeAllocDecimal ( double value )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return this.typeAllocDecimal( outType, value ) != NOERR ? null : outType;
	}
	
/**
	Allocates a new instance of type <i>decimal</i> with the given precision.

	@param	outDecimal	the resulting new type instance
	@param	value	the double value which the new type instance will hold
	@param	precision	the decimal precision that the new type instance will output to
	
	@blueworld
*/
	//osError lasso_typeAllocDecimal ( lasso_request_t token, lasso_type_t * outDecimal, double value );
	public native int typeAllocDecimal ( LassoTypeRef outDecimal, double value, int precision );

/**
	Allocates a new instance of type <i>decimal</i> with the given precision.
	
	@param	value		the double value which the new type instance will hold
	@param	precision	the decimal precision that the new type instance will output to
	@return 			the resulting new type instance, or <code>null</code> on error
		
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef typeAllocDecimal ( double value, int precision )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return this.typeAllocDecimal( outType, value, precision ) != NOERR ? null : outType;
	}

/**
	Allocates a new instance of type <i>pair</i>.
	
	@param	outPair		the resulting new type instance
	@param	inFirst		the instance to use for the first member of the pair. A reference to the instance will be made.
	@param	inSecond	the instance to use for the second member of the pair. A reference to the instance will be made.
	
	@blueworld
*/
	//osError lasso_typeAllocPair ( lasso_request_t token, lasso_type_t * outPair, lasso_type_t first, lasso_type_t second );
	public native int typeAllocPair ( LassoTypeRef outPair, LassoTypeRef inFirst, LassoTypeRef inSecond);

/**
	Allocates a new instance of type <i>pair</i>.
	
	@param	inFirst		the instance to use for the first member of the pair. A reference to the instance will be made.
	@param	inSecond	the instance to use for the second member of the pair. A reference to the instance will be made.
	@return 			the resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocPair ( LassoTypeRef inFirst, LassoTypeRef inSecond )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return this.typeAllocPair( outType, inFirst, inSecond ) != NOERR ? null : outType;
	}
	
/**
 	Allocates a new hard reference to a type instance. The new instance will point to the original.
	
	@param	outRef		the resulting new type instance
	@param	referenced	the instance to be referenced
	
	@blueworld
*/
	//osError lasso_typeAllocReference( lasso_request_t token, lasso_type_t * outRef, lasso_type_t referenced );
	public native int typeAllocReference ( LassoTypeRef outRef, LassoTypeRef referenced );
	
/**
 	Allocates a new hard reference to a type instance. The new instance will point to the original.
	
	@param		referenced	the instance to be referenced
	@return 	the resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocReference ( LassoTypeRef referenced )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocReference( outType, referenced ) != NOERR) ? null : outType;
	}
	
/**
	Allocates a new instance of type <i>tag</i>.
 	<i>Note:</i> The method referred to by <code>className</code> and <code>methodName</code> parameters 
 	should have the same signature as {@link LassoTagModule#TAG_METHOD_PROTOTYPE TAG_METHOD_PROTOTYPE}.
	
	@param	outTag		the resulting new type instance
	@param	className	the name of the java class implementing your module
	@param	methodName	the name of a java method which will be called when the tag is used
	
	@blueworld
 */
	//osError lasso_typeAllocTag ( lasso_request_t token, lasso_type_t * outTag, lasso_tag_func nativeTagFunction );
	public native int typeAllocTag ( LassoTypeRef outTag, String className, String methodName );

/**
	Allocates a new instance of type <i>tag</i>.
 	<i>Note:</i> The method referred to by <code>className</code> and <code>methodName</code> parameters 
 	should have the same signature as {@link LassoTagModule#TAG_METHOD_PROTOTYPE TAG_METHOD_PROTOTYPE}.
	
	@param	className	the name of the java class implementing your module
	@param	methodName	the name of a java method which will be called when the tag is used
	@return				the	resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
 */
	private LassoTypeRef typeAllocTag ( String className, String methodName )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocTag( outType, className, methodName ) != NOERR) ? null : outType;
	}

/**
	Allocates a new instance of type tag from the given source text.
	
	@param	outTag	the resulting new type instance
	@param	source	the source text which will be compiled to become the body of the tag

	@since v8.0
	
	@blueworld
*/
	//osError lasso_typeAllocTagFromSource ( lasso_request_t token, lasso_type_t * outTag, const char * source, int length );
	public native int typeAllocTagFromSource ( LassoTypeRef outTag, String source );

/**
	Allocates a new instance of type tag from the given source text.
	
	@param	source	the source text which will be compiled to become the body of the tag
	@return 		the resulting new type instance, or <code>null</code> on error
	
	@since v8.0
	
 	@omnipilot
*/
	private LassoTypeRef typeAllocTag ( String source )
	{
		LassoTypeRef outTag = new LassoTypeRef();
		return (this.typeAllocTagFromSource( outTag, source ) != NOERR) ? null : outTag;
	}
	
/**
	Allocates a new instance of type <i>array</i>.
 
 	@param	outArray	the resulting new type instance
	@param	inElements	an array of <code>LassoTypeRef</code>'s which will become the elements of the array (each item will be referenced). Can be null
	
	@blueworld
 */
	//osError lasso_typeAllocArray ( lasso_request_t token, lasso_type_t * outArray, int count, lasso_type_t * elements );
	public native int typeAllocArray ( LassoTypeRef outArray, LassoTypeRef[] inElements);

/**
	Allocates a new instance of type <i>array</i>.
 	
	@param	inElements	an array of <code>LassoTypeRef</code>'s which will become the elements of the array (each item will be referenced). Can be null
 	@param				the	resulting new type instance, or <code>null</code> on error
 	
	@since v8.0
	
 	@omnipilot
 */
 	private LassoTypeRef typeAllocArray ( LassoTypeRef[] inElements )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocArray( outType, inElements ) != NOERR) ? null : outType;
	}
	
/** 
 	Allocates a new instance of type <i>map</i>.
	
	Note that count here for the map must be divisible by 2. Every two elements are taken to be the key and value, respectively.
 
 	@param	outMap		the resulting new type instance
	@param	inElements	an array of <code>LassoTypeRef</code>'s which will become the elements of the map. Each even numbered item will be treated as the key for following element
	
	@blueworld
 */
	//osError lasso_typeAllocMap ( lasso_request_t token, lasso_type_t * outMap, int count, lasso_type_t * elements );
	public native int typeAllocMap ( LassoTypeRef outMap, LassoTypeRef[] inElements );

/** 
 	Allocates a new instance of type <i>map</i>.
	
	Note that count here for the map must be divisible by 2. Every two elements are taken to be the key and value, respectively.
 
	@param	inElements	an array of <code>LassoTypeRef</code>'s which will become the elements of the map. Each even numbered item will be treated as the key for following element
 	@return 			the resulting new type instance, or <code>null</code> on error
 	
	@since v8.0
	
 	@omnipilot
 */
 	private LassoTypeRef typeAllocMap ( LassoTypeRef[] inElements )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocMap( outType, inElements ) != NOERR) ? null : outType;
	}
	
/**
	Allocates a new instance of type <i>map</i>.
	
 	Same as {@link #typeAllocMap(LassoTypeRef,LassoTypeRef[]) typeAllocMap}, 
 	only the map keys and values are passed as separate parameters.
 
  	@param	outMap	the resulting new type instance
 	@param	inKeys	an array of <code>LassoTypeRef</code>'s which will become the map keys
  	@param	inKeys	an array of <code>LassoTypeRef</code>'s which will become the map keys
 	
 */
	public native int typeAllocMap ( LassoTypeRef outMap, LassoTypeRef[] inKeys, LassoTypeRef[] inValues );

/**
	Allocates a new instance of type <i>map</i>.<br>
	
 	Same as {@link #typeAllocMap(LassoTypeRef,LassoTypeRef[],LassoTypeRef[]) typeAllocMap}, 
 	only the map keys and values are being passed here as separate parameters.
 	
 	@param	inKeys		an array of <code>LassoTypeRef</code>'s which will become the map keys
  	@param	inValues	an array of <code>LassoTypeRef</code>'s which will become the map values
 	@return 			the resulting new type instance, or <code>null</code> on error
 
 	@since v8.0
 	
 	@omnipilot
 */
  	private LassoTypeRef typeAllocMap ( LassoTypeRef[] inKeys, LassoTypeRef[] inValues )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocMap( outType, inKeys, inValues ) != NOERR) ? null : outType;
	}
	
/**
	Allocates a new instance of type <i>boolean</i>.
  
  	@param	outBool	the resulting new type instance
	@param	value	the boolean value which the new type instance will hold
	
	@blueworld
*/
	//osError lasso_typeAllocBoolean( lasso_request_t token, lasso_type_t * outBool, bool inValue );
	public native int typeAllocBoolean( LassoTypeRef outBool, boolean value );

/**
	Allocates a new instance of type <i>boolean</i>.

	@param	value	the boolean value which the new type instance will hold
  	@return 		the resulting new type instance, or <code>null</code> on error
  	
  	@omnipilot
*/
  	private LassoTypeRef typeAllocBoolean ( boolean value )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.typeAllocBoolean( outType, value ) != NOERR) ? null : outType;
	}


/**
	This function transfers the data from one type to another type. Both types must be valid and pre-allocated. 
	After the call, victim will still be valid, but will be of type null.
	@since v7.0
*/
public native int typeStealValue( LassoTypeRef inThief, LassoTypeRef inVictim );

/**
	The basis of table-level automatic encoding and translation.
	Converts a Lasso type INTO single-byte or binary data using the specific encoding name.
	The default for all database, column, table names should be "iso8859-1"
	
	@since v7.0
*/
//osError lasso_handleExternalConversion(lasso_request_t token, lasso_type_t instance, const char * encoding, auto_lasso_value_t * outVal);
public native byte[] handleExternalConversion(LassoTypeRef inInstance, String inEncoding);

/**
	Converts a single-byte or binary representation of a Lasso type BACK INTO an instance of that type
	
	@param	inClosestLassoType	desired Lasso type. pass one of the TYPE_XXX constants defined in {@link LassoValue LassoValue}
	@param	outType	resulting Lasso type intance
	
	@since v7.0	
*/
//osError lasso_handleInternalConversion(lasso_request_t token, const char * src, unsigned int srcLen, const char * encoding, LP_TypeDesc closestLassoType, lasso_type_t * outType);
public native int handleInternalConversion( byte[] inData, String inEncoding, int inClosestLassoType, LassoTypeRef outType );



/*********************************************************
TYPE ACCESSORS
**********************************************************/

/*
	Getting or Setting Values of Built-in Type Instances.
	
	The following functions get and set the data on a previously created built-in type instance.
	When getting a value, the source type instance will not be altered.
	When setting a value, the provided type instance is converted, if required.
	The LassoCall token may be null.
*/

/**
  	Retrieves character data from a type instance.
	
	If the type is a string instance, the data will be converted to UTF-8.<br>
	If the type is a bytes instance, the data will be provided unaltered.<br>
	For any other type, the result will be the same as converting the type into a string and returning the data as UTF-8
	
	@param	type		the type instance from which to retrieve the character data
	@param	outValue	on return, the data will be stored in the <code>name</code> field of the <code>LassoValue</code>
	
	@blueworld
 */
	//osError lasso_typeGetString( lasso_request_t token, lasso_type_t type, auto_lasso_value_t * val );
	public native int typeGetString( LassoTypeRef type, LassoValue outValue );

/**
  	Retrieves character data from a type instance.
	
	If the type is a string instance, the data will be converted to UTF-8.<br>
	If the type is a bytes instance, the data will be provided unaltered.<br>
	For any other type, the result will be the same as converting the type into a string and returning the data as UTF-8
	
	@param	type	the type instance from which to retrieve the character data
	@return 		the resulting string, or <code>null</code> on error
	
	@since v8.0
	
	@omnipilot
 */
	private String typeGetString( LassoTypeRef type )
	{
		LassoValue outValue = new LassoValue();
		return (this.typeGetString( type, outValue ) != NOERR) ? null : outValue.name();		
	}
	
/**
	Retrieves data from a type instance as an array of bytes.

	@param	type	the type instance from which to retrieve the byte data
	@return 		the resulting byte array
	
	@since v7.0
*/
	public native byte[] typeGetBytes( LassoTypeRef type );

/**
  	Retrieves the integer value from a type instance.
	
	@param	type		the type instance from which to retieve the integer value
	@param	outValue	on return, contains the resulting 64-bit integer value
	
	@blueworld
	
 */
	//osError lasso_typeGetInteger( lasso_request_t token, lasso_type_t type, osInt64 * out );
	public native int typeGetInteger( LassoTypeRef type, IntValue outValue );

/**
  	Retrieves the integer value from a type instance.
	
	@param	type	the type instance from which to retieve the integer value
	@return 		the resulting 64-bit integer value
	
	@since v8.0
	
	@omnipilot	
 */
	private long typeGetInteger( LassoTypeRef type )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (this.typeGetInteger( type, outValue ) != NOERR) ? 0 : outValue.longValue();	
	}
	
/**
	Retrieves the decimal value from a type instance.
	
	@param	type		the type instance from which to retieve the decimal value
	@param	outValue	on return, the resulting double value

	@blueworld
 */
	//osError lasso_typeGetDecimal( lasso_request_t token, lasso_type_t type, double * out );
	public native int typeGetDecimal( LassoTypeRef type, FloatValue outValue );

/**
	Retrieves the decimal value from a type instance.
	
	@param	type	the type instance from which to retieve the decimal value
	@return 		on return, the resulting double value
	
	@since v8.0
	
	@omnipilot	
 */
 	private double typeGetDecimal( LassoTypeRef type )
	{
		com.blueworld.lassopro.FloatValue outValue = new com.blueworld.lassopro.FloatValue();
		return (this.typeGetDecimal( type, outValue ) != NOERR) ? 0 : outValue.doubleValue();	
	}
	
/**
 	Retrieves the boolean value from a type instance.
 	
  	@param 	type 	the type instance from which to retieve the boolean value.
	@param	out		on return, the resulting boolean value
	
	@blueworld	
 */
	//osError lasso_typeGetBoolean( lasso_request_t token, lasso_type_t type, bool * out );
	public native int typeGetBoolean( LassoTypeRef type, IntValue outValue );

/**
	Retrieves the boolean value from a type instance.
	
	@param	type	the type instance from which to retieve the decimal value
	@return 		on return, the resulting boolean value
	
	@since v8.0
	
	@omnipilot	
 */
 	private boolean typeGetBoolean( LassoTypeRef type )
	{
		com.blueworld.lassopro.IntValue outValue = new com.blueworld.lassopro.IntValue();
		return (this.typeGetBoolean( type, outValue ) != NOERR) ? false : outValue.booleanValue();	
	}	
	
/**
	Converts the type instance into a string and sets the value.
	
	@param	type	the type instance to set
	@param	value	the source string data
 */
	//osError lasso_typeSetString( lasso_request_t token, lasso_type_t type, const char * val, int len );
	public native int typeSetString( LassoTypeRef type, String value );

/**
	Converts the type instance into a string and sets the value.
	
	@param	type	the type instance to set
	@param	value	the data
	
	@since v7.0
*/
	//osError lasso_typeSetString( lasso_request_t token, lasso_type_t type, const char * val, int len );
	public native int typeSetBytes( LassoTypeRef type, byte[] value );

/**
	Converts the type instance using the specified source data and conversion method.
	If the special conversion method of "BINARY" is used, the type instance will be converted into a bytes type, the contents of which will be the unaltered source data.
	
	The remaining possible values for the conversion method are any of the character encoding methods supported by ICU or any of the converters 
	stored in the '__internal_converters__' global variable.
	
	If any of the built-in ICU conversion methods are used, the provided instance will be converted into a string type.
	If any other conversion methods are used, the resulting type of the provided instance can can vary.
	
	@param	type	the type instance to be converted
	@param	value	the source data to be used for the conversion
	@param	conv	the conversion method to use when transforming type instance
	
	@since v8.0
*/
	//osError lasso_typeSetStringConv( lasso_request_t token, lasso_type_t type, const char * val, int len, const char * conv );
	public native int typeSetStringConv( LassoTypeRef type, byte[] value, String conv );

/**
	Appends additional data to the string instance.
	
	@param	type	the type instance which will be appended to
	@param	value	the source data to append
	
	@since v7.0
*/
	//osError lasso_typeAppendString( lasso_request_t token, lasso_type_t type, const char * val, int len );
	public native int typeAppendString( LassoTypeRef type, String value );

/**
	Sets the integer value of a type instance, converting the type if necessary.
	
	@param	type	the type instance which will be set to the provided integer
	@param	value	the integer value to use
 */
	//osError lasso_typeSetInteger( lasso_request_t token, lasso_type_t type, osInt64 val );
	public native int typeSetInteger( LassoTypeRef type, long value );

/**
	Sets the decimal value of a type instance, converting the type if necessary.
	
	@param	type	the type instance which will be set to the provided double
	@param	value	the double value to use
 */
	//osError lasso_typeSetDecimal( lasso_request_t token, lasso_type_t type, double val );
	public native int typeSetDecimal( LassoTypeRef type, double value );

/**
	Sets the decimal value of a type instance, converting the type if necessary.
  
  	@param	type 		the type instance which will be set to the provided integer
	@param	value		the double value to use
	@param	precision	the decimal precision which the decimal instance will output to
 */
	//osError lasso_typeSetDecimal( lasso_request_t token, lasso_type_t type, double val, int precision );
	public native int typeSetDecimal( LassoTypeRef type, double value, int precision );

/**
	Sets the boolean value of a type instance, converting the type if necessary.
	
	@param	type	the type instance which will be set to the provided boolean
	@param	value	the boolean value to use.
 */
	//osError lasso_typeSetBoolean( lasso_request_t token, lasso_type_t type, bool val );
	public native int typeSetBoolean( LassoTypeRef type, boolean value );

// Array

/**
	Retrieves the size of the provided array instance.
	
	@param	array	the array instance
	@param	outLen	on return, contains the resulting size
	
	@blueworld
 */
	//osError lasso_arrayGetSize( lasso_request_t token, lasso_type_t array, int * len );
	public native int arrayGetSize( LassoTypeRef array, IntValue outLen );

/**
	Retrieves the size of the provided array instance.
	
	@param	array	the array instance
	@return			the resulting array size, or <code>-1</code> on error
	
	@since v8.0
	
	@omnipilot
 */
	private int arrayGetSize( LassoTypeRef array )
	{
		com.blueworld.lassopro.IntValue outLen = new com.blueworld.lassopro.IntValue();
		return (this.arrayGetSize( array, outLen ) != NOERR) ? -1 : outLen.intValue();	
	}

/**
	Retrieves the specified element of the provided array instance.
	
	@param	array		the array instance
	@param	index		the zero-based index at which to retreive the element
	@param	outElement	the resulting element

	@blueworld
 */
	//osError lasso_arrayGetElement( lasso_request_t token, lasso_type_t array, int index, lasso_type_t * out );
	public native int arrayGetElement( LassoTypeRef array, int index, LassoTypeRef outElement );

/**
	Retrieves the specified element of the provided array instance.
	
	@param	array		the array instance
	@param	index		the zero-based index at which to retreive the element
	@return				the resulting array element, or <code>null</code> on error
	
	@since v8.0
	
	@omnipilot
 */
	private LassoTypeRef arrayGetElement( LassoTypeRef array, int index )
	{
		LassoTypeRef outElement = new LassoTypeRef();
		return (this.arrayGetElement( array, index, outElement ) != NOERR) ? null : outElement;	
	}	
/**
	Sets the specified element of the provided array instance.
	
	@param	array	the array instance
	@param	index	the zero-based index at which to set the element
	@param	element	the type instance which will be placed at the specified index. The instance will be referenced
 */
	//osError lasso_arraySetElement(lasso_request_t token, lasso_type_t array, int index, lasso_type_t elem );
	public native int arraySetElement( LassoTypeRef array, int index, LassoTypeRef element );

/**
	Removes the specified element from the provided array instance.
	
	@param	array	the array instance
	@param	index	the zero based index at which to remove
 */
	//osError lasso_arrayRemoveElement( lasso_request_t token, lasso_type_t array, int index );
	public native int arrayRemoveElement( LassoTypeRef array, int index );

// Map

/**
	Retrieves the size of the provided map instance.
	
	@param	map		the map instance
	@param	outLen	on return, contains the resulting map size
	
	@blueworld
 */
	//osError lasso_mapGetSize( lasso_request_t token, lasso_type_t mp, int * len );
	public native int mapGetSize( LassoTypeRef map, IntValue outLen );

/**
	Retrieves the size of the provided map instance.
	
	@param	map		the map instance
	@return			the resulting map size, or <code>-1</code> on error
	
	@since v8.0
	
	@omnipilot
 */
	private int mapGetSize( LassoTypeRef map )
	{
		com.blueworld.lassopro.IntValue outLen = new com.blueworld.lassopro.IntValue();
		return (this.mapGetSize( map, outLen ) != NOERR) ? -1 : outLen.intValue();	
	}
	
/**
	Searches for the specified key in the provided map instance.
	
	@param	map			the map instance
	@param	key			the key to search for within the map
	@param	outElement	on return, the found map element
	
	@blueworld
 */
	//osError lasso_mapGetElement( lasso_request_t token, lasso_type_t mp, lasso_type_t key, lasso_type_t * out );
	public native int mapFindElement( LassoTypeRef map, LassoTypeRef key, LassoTypeRef outElement );

/**
	Searches for the specified key in the provided map instance.
	
	@param	map		the map instance
	@param	key		the key to search for within the map
	@return			a map element, or <code>null</code> if no value exists for this key
	
	@since v8.0
	
	@omnipilot
 */
	private LassoTypeRef mapFindElement( LassoTypeRef map, LassoTypeRef key )
	{
		LassoTypeRef outElement = new LassoTypeRef();
		return (this.mapFindElement( map, key, outElement ) != NOERR) ? null : outElement;	
	}	
	
/**
	Retrieves the specified element of the provided map instance by index.
	
	@param	map		the map instance
	@param	index	the zero based index at which to retreive
	@param	outPair	the pointer in which to store the result. The result will be a pair whose first member is the key and second member is the associated element
	
	@blueworld
*/
	//osError lasso_mapGetNthElement( lasso_request_t token, lasso_type_t mp, int index, lasso_type_t * outPair );
	public native int mapGetElement( LassoTypeRef map, int index, LassoTypeRef outPair );

/**
	Retrieves the specified element of the provided map instance by index.
	
	@param	map		the map instance
	@param	index	the zero based index at which to retreive
	@return			a pair whose first member is the key and second member is the associated element, or <code>null</code> if <code>index</code> is invalid

	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef mapGetElement( LassoTypeRef map, int index )
	{
		LassoTypeRef outElement = new LassoTypeRef();
		return (this.mapGetElement( map, index, outElement ) != NOERR) ? null : outElement;	
	}
	
/**
	Sets the specified element of the provided map instance.
	
	If no elements were previously stored under the specified <code>key</code>, 
 	the element will be added to the map, otherwise the old element will be replaced by a new <code>value</code>.
 
	@param	map		the map instance
	@param	key		the instance of the key which will be set
	@param	value	the value of the element to associate with the provided key
 */
	//osError lasso_mapSetElement( lasso_request_t token, lasso_type_t mp, lasso_type_t key, lasso_type_t value );
	public native int mapSetElement( LassoTypeRef map, LassoTypeRef key, LassoTypeRef value );

/**
	Removes the specified element from the map based on the provided key.
	
	@param	map	the map instance
	@param	key	the key to be removed
*/
	//osError lasso_mapRemoveElement( lasso_request_t token, lasso_type_t mp, lasso_type_t key );
	public native int mapRemoveElement( LassoTypeRef map, LassoTypeRef key );


// Pair

/**
	Retrieves the first member from the provided pair.
	
	@param	pair		the pair instance
	@param	outValue	on return, the resulting type instance
	
	@blueworld
 */
	//osError lasso_pairGetFirst( lasso_request_t token, lasso_type_t pr, lasso_type_t * out );
	public native int pairGetFirst( LassoTypeRef pair, LassoTypeRef outValue );

/**
	Retrieves the first member from the provided pair.
	
	@param	pair	the pair instance
	@return			the resulting type instance, or <code>null</code> on error

	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef pairGetFirst( LassoTypeRef pair )
	{
		LassoTypeRef outValue = new LassoTypeRef();
		return (this.pairGetFirst( pair, outValue ) != NOERR) ? null : outValue;	
	}
	
/**
	Retrieves the second member from the provided pair.
	
	@param	pair	the pair instance
	@return			the resulting type instance, or <code>null</code> on error
	
	@blueworld
 */
	//osError lasso_pairGetSecond( lasso_request_t token, lasso_type_t pr, lasso_type_t * out );
	public native int pairGetSecond( LassoTypeRef pair, LassoTypeRef outValue );

/**
	Retrieves the first member from the provided pair.
	
	@param	pair	the pair instance
	@return			the resulting type instance, or <code>null</code> on error

	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef pairGetSecond( LassoTypeRef pair )
	{
		LassoTypeRef outValue = new LassoTypeRef();
		return (this.pairGetSecond( pair, outValue ) != NOERR) ? null : outValue;	
	}
	
/**
	Sets the first member of the provided pair.
	
	@param	pair	the pair instance
	@param	first	the instance to use as the first member. The instance will be referenced
 */
	//osError lasso_pairSetFirst( lasso_request_t token, lasso_type_t pr, lasso_type_t frst );
	public native int pairSetFirst( LassoTypeRef pair, LassoTypeRef first );

/**
	Sets the second member of the provided pair.
	
	@param	pair	the pair instance
	@param	second	the instance to use as the second member. The instance will be referenced
 */
	//osError lasso_pairSetSecond( lasso_request_t token, lasso_type_t pr, lasso_type_t scnd );
	public native int pairSetSecond( LassoTypeRef pair, LassoTypeRef second );
	
	
/*********************************************************
TYPE PROPERTY MAPS
**********************************************************/
/*
  The properties of a type is a map containing all the members belonging to the type. 
  The map accounts for inheritance by having an item keyed by <code>__PARENT_CONTEXT__</code>
  which is a reference to the properties of the next closest ancestor. 
  Each item in a property map is, of course, keyed and the items are accessed using functions listed below.

  If <code>targetType</code> is NULL, the the symbol map belonging to the current execution context will be returned.
*/

/** This method has two uses. If the <code>targetType</code> parameter is not <code>null</code>, 
	it is used to get all the data and tag members from a given type. 
	They are returned as an array of pairs in the <code>outPair</code> value.
	The first element of each pair is the map of data members for the type. The second element is the map of tag members.
	Each element in the array represents the members of each type inherited by the <code>targetType</code>.
	
	<p>If the <code>targetType</code> parameter is <code>null</code>, <code>typeGetProperties</code> 
	will return an array containing the variable maps for the currently active request.
 */
	//osError lasso_typeGetProperties ( lasso_request_t token, lasso_type_t targetType, lasso_type_t * outPair );
	public native int typeGetProperties ( LassoTypeRef targetType, LassoTypeRef outPair );

/*********************************************************
	TYPE INHERITANCE
**********************************************************/

/** 
	Change the inheritence structure of a type.
	
	Sets new parent of a child.
	Any parent that child currently has will be destroyed.
	
	@param	inChild		the child instance which will have its inheritance changed
	@param	inNewParent	the new parent for the child
	
	@since v7.0
*/
	//osError lasso_typeInheritFrom( lasso_request_t token, lasso_type_t child, lasso_type_t newParent );
	public native int typeInheritFrom( LassoTypeRef inChild, LassoTypeRef inNewParent );

/**
	Retrieves the parent of the provided type instance.
	
	@param	from		the child instance from which to retrieve the parent
	@param	outParent	on return, the type parent
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_typeGetParent( lasso_request_t token, lasso_type_t from, lasso_type_t * outParent );
	public native int typeGetParent( LassoTypeRef from, LassoTypeRef outParent );

/**
	Retrieves the parent of the provided type instance.
	
	@param	from	the child instance from which to retrieve the parent
	@return			the type parent, or <code>null</code> on error
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef typeGetParent( LassoTypeRef from )
	{
		LassoTypeRef outValue = new LassoTypeRef();
		return (this.typeGetParent( from, outValue ) != NOERR) ? null : outValue;	
	}

/**
	Retrieves the child of the provided instance.
	
	@param	from		the parent instance from which to retrieve the child
	@param	outChild 	on return, the resulting child type
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_typeGetChild( lasso_request_t token, lasso_type_t from, lasso_type_t * outChild );
	public native int typeGetChild( LassoTypeRef from, LassoTypeRef outChild );

/**
	Retrieves the child of the provided instance.
	
	@param	from	the child instance from which to retrieve the parent
	@return			the type child, or <code>null</code> on error
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef typeGetChild( LassoTypeRef from )
	{
		LassoTypeRef outValue = new LassoTypeRef();
		return (this.typeGetChild( from, outValue ) != NOERR) ? null : outValue;	
	}
	
/**
	Adds a member to a type instance.
	
	If the new member is a tag, it will be added to the tag members for the type. Otherwise, the new member will be added as a data member.
	
	@param	toType	the type instance to which the new member will be added
	@param	named	the name for the new member
	@param	member	the new member to add
 */
	//osError lasso_typeAddMember( lasso_request_t token, lasso_type_t to, const char * named, lasso_type_t member );
	public native int typeAddMember( LassoTypeRef toType, String named, LassoTypeRef member );

/**
	Retrieves a member from a type instance.
	
	Members are searched for by name with tag members being searched first.
	Data members are searched if no tag is found with the given name.
	
	@param	fromType	the type instance in which to search
	@param	named		the name of the member to look for
	@param	outMember 	on return, the found member
	
	@blueworld
 */
	//osError lasso_typeGetMember( lasso_request_t token, lasso_type_t from, const char * named, lasso_type_t * out );
	public native int typeGetMember( LassoTypeRef fromType, String named, LassoTypeRef outMember );

/**
	Retrieves a member from a type instance.
	
	Members are searched for by name with tag members being searched first.
	Data members are searched if no tag is found with the given name.
	
	@param	fromType	the type instance in which to search
	@param	named		the name of the member to look for
	@return				the resulting type member, or <code>null</code> if not found
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef typeGetMember( LassoTypeRef from, String named )
	{
		LassoTypeRef outMember = new LassoTypeRef();
		return (this.typeGetMember( from, named, outMember ) != NOERR) ? null : outMember;	
	}
	
/**
	Adds a member tag to a type instance. 
	A shortcut for the typeAllocTag/typeAddMember pair.

	@since v7.0
 */
	//Not available in LCAPI
	public native int typeAddMemberTag( String className, String methodName, LassoTypeRef toType, String tagName );


/*********************************************************
	FLAGS
**********************************************************/

/**
	Sets the flag for the given type instance.
	
	@param	type	the type instance on which to set the flag
	@param	flag	the flag to set
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_typeSetFlag(lasso_request_t token, lasso_type_t type, int flag);
	public native int typeSetFlag( LassoTypeRef type, int flag );

/**
	Tests the flag for the given type instance.
	
	Returns <code>true</code> if the flag is set for that type instance.
	
	@param	type	the type instance to be tested
	@param	flag	the flag to test for
	
	@since v8.0
	
	@omnipilot	
*/
	//osError lasso_typeHasFlag(lasso_request_t token, lasso_type_t type, int flag);
	public native int typeHasFlag( LassoTypeRef type, int flag );

/**
	Removes the flag for the given type instance.
	
	@param	type	the type instance
	@param	flag	the flag to remove from the type instance
	
	@since v8.0
	
	@omnipilot		
*/
	//osError lasso_typeClearFlag(lasso_request_t token, lasso_type_t type, int flag);
	public native int typeClearFlag( LassoTypeRef type, int flag );	
	
/*********************************************************
TYPE NAME
**********************************************************/
/**
 Retrieves the name of the <code>target</code> type.
 */
//osError lasso_typename( lasso_request_t token, lasso_type_t target, auto_lasso_value_t * outName );
public native int typename( LassoTypeRef target, LassoValue outName );



/*********************************************************
	GETTING "self"
**********************************************************/

/**
	Returns the type instance that the current tag call was a member of.
	
	This is used in member tags of custom types to return the target of the current call.
	
	@param	outSelf	on return, the resulting type instance
	
	@blueworld
*/
	//osError lasso_getTagSelf ( lasso_request_t token, lasso_type_t * self );
	public native int getTagSelf ( LassoTypeRef outSelf );

/**
	Returns the type instance that the current tag call was a member of.
	
	This is used in member tags of custom types to return the target of the current call.
	
	@return the resulting type instance, or <code>null</code> on error

	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef getTagSelf ( )
	{
		LassoTypeRef out = new LassoTypeRef();
		return (this.getTagSelf(out) != NOERR) ? null : out;
	}

/**
	Returns the name of the target type.
	
	@param	target	the type instance to get the name of
	@param	outName	the resulting name which will be stored in the <code>name</code> portion of the <code>LassoValue</code>
	
	@blueworld	
*/	
	//osError lasso_typeGetName( lasso_request_t token, lasso_type_t target, auto_lasso_value_t * outName );
	public native int typeGetName( LassoTypeRef target, LassoValue outName );
	
/**
	Returns the name of the target type.
	
	@param	target	the type instance to get the name of
	@return 		the resulting type name, or <code>null</code> on error
	
	@since v8.0
	
	@omnipilot
*/
	private String typeGetName ( LassoTypeRef target )
	{
		LassoValue outName = new LassoValue();
		return (this.typeGetName(target, outName) != NOERR) ? null : outName.name();
	}
	
/**
	Tests to see if a type is an instance of another type.
	
	Test by type code.
	
	@param	target		the target instance to test
	@param	typeName	the type code to test for
*/
	//osError lasso_typeIsA2( lasso_request_t token, lasso_type_t target, const UChar * typeName );
	public native int typeIsA( LassoTypeRef target, String typeName );

/**
	Tests to see if a type is an instance of another type.
	
	Test based on another type instance.
	
	@param	target	the target instance to test
	@param	type	the type name to test for
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_typeIsA3( lasso_request_t token, lasso_type_t target, lasso_type_t type );
	public native int typeIsA( LassoTypeRef target, LassoTypeRef type );
	
/*********************************************************
	RETURN VALUES
**********************************************************/
/**
	Specifies the return value of the current LJAPI tag call.

	Any type can be returned. A tag can only have one return value. Setting another return value will overwrite the previous.
	
	@param	token	the request token which should be passed to subsequent LJAPI methods for the duration of the current service function call
	@param	value	the value to return
*/
	//osError lasso_returnTagValue ( lasso_request_t token, lasso_type_t value );
	public native int returnTagValue ( LassoTypeRef value );

/**
	Return a boolean value from the current LJAPI tag call.
	
	@param	value	the boolean value to return
	
	@since v8.0
	
	@omnipilot
*/
	//osError lasso_returnTagValueBoolean( lasso_request_t token, bool b );
	public int returnTagValue ( boolean value )
	{
		return this.returnTagValue( this.typeAllocBoolean( value ) );
	}

/**
	Return an integer value from the current LJAPI tag call.
	
	@param	value	the integer value to return
	
	@since v8.0
*/
	//osError lasso_returnTagValueInteger( lasso_request_t token, osInt64 i );
	public int returnTagValue ( long value )
	{
		return this.returnTagValue( this.typeAllocInteger( value ) );
	}
	
/**
	Return a string value from the current LJAPI tag call.
	
	@param	value	the string data to be returned
	
	@since v8.0
*/
	//osError lasso_returnTagValueString( lasso_request_t token, const char * p, int l );	
	public int returnTagValue ( String value )
	{
		return this.returnTagValue( this.typeAllocString( value ) );
	}
	
/**
	Return a decimal value from the current LJAPI tag call.
	
	@param	value	the double value to return
*/
	//osError lasso_returnTagValueDecimal( lasso_request_t token, double d );
	public int returnTagValue ( double value )
	{
		return this.returnTagValue( this.typeAllocDecimal( value ) );
	}

/**
	Return binary data from the current LJAPI tag call.
	
	When using this, the result of the LJAPI call will be a bytes type.
	This can be called as many times as needed and new data will be appended to any previous data.

	@param	data	the binary data to return
*/
	//osError lasso_returnTagValueBytes( lasso_request_t token, const char * data, int length );
	public int returnTagValue ( byte[] value )
	{
		LassoTypeRef out = new LassoTypeRef();
		int err = this.typeAllocString( out, value );
		return (err != NOERR) ? err : this.returnTagValue( out );
	}

/**
	Provides direct access to the LJAPI tag's return value.
	
	In many cases, directly manipulating this value can yield better performance than {@link #returnTagValue returnTagValue}.
	The resulting type instance will start out as type <code>null</code>.
	
	@param	outValue	the resulting return value type instance
	
	@since v8.0
	
	@blueworld
*/
	//osError lasso_getTagReturnValue( LassoTypeRef outValue );
	public native int getTagReturnValue( LassoTypeRef outValue );

/**
	Provides direct access to the LJAPI tag's return value.
	
	In many cases, directly manipulating this value can yield better performance than {@link #returnTagValue returnTagValue}.	
	The resulting type instance will start out as type <code>null</code>.
	
	@return	the resulting return value type instance, or <code>null</code> on error
	
	@since v8.0
	
	@omnipilot
*/		
	private LassoTypeRef getTagReturnValue( )
	{
		LassoTypeRef out = new LassoTypeRef();
		return ( this.getTagReturnValue(out) != NOERR ) ? null : out;
	}

/*********************************************************
	EXECUTING A MEMBER OR TAG
**********************************************************/

 /**
	Execute a given tag.
	
	The tag can be run given a specific name and an array of parameters. 
	The tag's return value can be accessed.
	If the tag is a member tag, the instance of which it is a member can be passed using the final parameter.
	The params, returnValue and optionalTarget parameters may all be <code>null</code>.
	
	@param	tagType 		the tag instance to be executed
	@param	named			the name for the tag
	@param	params			the array of parameters for the tag. May be <code>null</code>
	@param	returnValue		on return, contains the resulting value.
	@param	optionalTarget	the target instance for a member tag call. May be <code>null</code> if the tag is not a member tag
*/
	//osError lasso_typeRunTag ( lasso_request_t token, lasso_type_t tagType, int paramCount, lasso_type_t ** params, lasso_type_t * returnValue, lasso_type_t optionalTarget );
	public native int typeRunTag ( LassoTypeRef tagType, String named, LassoTypeRef[] params, LassoTypeRef returnValue, LassoTypeRef optionalTarget );


/**
	Used to execute a given tag. 
	
	Convenience function. Same as {@link #typeRunTag typeRunTag}, except that is accepts a single <code>LassoTypeRef</code>
	parameter instead of an array, returning the resulting type instance.
	
	@param	tagType	must be a valid instance of an executable tag of type <code>LASSO_TAG</code>
	@param	named	optionally sets the tag name at the time of execution. Could be <code>null</code>
 	@param	param	Optional. Single tag parameter. Could be <code>null</code>.
 	@param	returnValue	the result of this tag execution
 	@param	optionalTarget	would be used in a member tag call. Otherwise it should be <code>null</code>
 	
 	@blueworld
 */
	public native int typeRunTag ( LassoTypeRef tagType, String named, LassoTypeRef param, LassoTypeRef returnValue, LassoTypeRef optionalTarget );

/**
	Execute a given tag.

	The tag can be run given a specific name and an array of parameters. 
	The tag's return value can be accessed.
	If the tag is a member tag, the instance of which it is a member can be passed using the final parameter.
	The params, returnValue and optionalTarget parameters may all be <code>null</code>.
	
	Simplified version of the {@link #typeRunTag(LassoTypeRef,String,LassoTypeRef[],LassoTypeRef,LassoTypeRef) typeRunTag},
	which accepts single <code>LassoTypeRef</code> parameter instead of an array, 
	and returns the resulting type instance (or <code>null</code>).
	
	@param	tagType			must be a valid instance of an executable tag of type <code>LASSO_TAG</code>
	@param	named			optionally sets the tag name at the time of execution. Could be <code>null</code>
 	@param	param			single tag parameter. Could be <code>null</code>
 	@param	optionalTarget	the target instance for a member tag call. May be <code>null</code> if the tag is not a member tag
 	
 	@since v8.0
 	
 	@omnipilot
 */
	private LassoTypeRef typeRunTag ( LassoTypeRef tagType, String named, LassoTypeRef param, LassoTypeRef optionalTarget )
	{
		LassoTypeRef out = new LassoTypeRef();
		return ( this.typeRunTag(tagType, named, param, out, optionalTarget) != NOERR ) ? null : out;	
	}
	
/**
	Searches for a tag.
	
	The search method is the same as would be used to find a tag in a script. Returns <code>ERR_NOERR</code> if the tag is found.
	The final parameter can be used to disable searching for the tag in any on-demand libraries.
	
	@param	named			the tag name to search for
	@param	outTag			the resulting tag, if found
	@param	disableOnDemand	used to disable the searching of on-demand libraries
	
	@since v8.0
	
	@blueworld
*/
	//osError lasso_findTagW( lasso_request_t token, const UChar* name, lasso_type_t * outTag, bool disableOnDemand );
	public native int findTag( String named, LassoTypeRef outTag, boolean disableOnDemand );
	
/**
	Searches for a tag.
	
	The search method is the same as would be used to find a tag in a script. 
	Returns valid {@link LassoTypeRef LassoTypeRef} if the tag is found.
	The final parameter can be used to disable searching for the tag in any on-demand libraries.
	
	@param	named			the tag name to search for
	@param	disableOnDemand	used to disable the searching of on-demand libraries
	@return					the resulting tag, or <code>null</code> if not found
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef findTag( String named, boolean disableOnDemand )
	{
		LassoTypeRef out = new LassoTypeRef();
		return ( this.findTag(named, out, disableOnDemand) != NOERR ) ? null : out;	
	}
	
/*********************************************************
	VARIABLES
**********************************************************/
/**
	Provides access to a defined variable.

	@param	named		the name of the variable to access
	@param	outValue	the resulting variable value
	
	@blueworld
*/
	//osError lasso_getVariable2( lasso_request_t token, lasso_type_t key, lasso_type_t * value );
	public native int getVariable2( String named, LassoTypeRef outValue );

/**
	Provides access to a defined variable.

	@param	named		the name of the variable to access
	@return				the resulting variable value, or <code>null</code> if not found
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef getVariable( String named )
	{
		LassoTypeRef outValue = new LassoTypeRef();
		return ( this.getVariable2( named, outValue ) != NOERR) ? null : outValue;
	}
	
/**
	Sets the specified variable's value.
	
	The variable does not have to already exist.
		
	@param	named	the name of the variable to set
	@param	value	the value to set
	
	@blueworld
*/
	//osError lasso_setVariable2( lasso_request_t token, lasso_type_t key, lasso_type_t value );
	public native int setVariable2( String named, LassoTypeRef value );

/**
	Sets the specified variable's value.
	
	The variable does not have to already exist.
		
	@param	named	the name of the variable to set
	@param	value	the value to set
	
	@since v8.0
	
	@omnipilot
*/
	private int setVariable( String named, LassoTypeRef value )
	{
		return this.setVariable2( named, value );
	}
	

/*********************************************************
	TAG PARAMS
**********************************************************/
/**
	Retrieves a parameter that was passed to the LJAPI tag call.

	The parameter index is zero-based.

	@param	paramIndex	the zero based index of the desired parameter
	@param	outType		the resulting parameter value
	
	@blueworld
*/
	//osError lasso_getTagParam2( lasso_request_t token, int paramIndex, lasso_type_t * result );
	public native int getTagParam2( int paramIndex, LassoTypeRef outType );

/**
	Retrieves a parameter that was passed to the LJAPI tag call.

	The parameter index is zero-based.

	@param	paramIndex	the zero based index of the desired parameter
	@return				the resulting parameter value (type instance), or <code>null</code> if <code>index</code> was invalid
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef getTagParam2( int paramIndex )
	{
		LassoTypeRef outValue = new LassoTypeRef();
		return ( this.getTagParam2( paramIndex, outValue ) != NOERR) ? null : outValue;	
	}
	
/**
	Retrieves a parameter that was passed to the LJAPI tag call.

	The parameter must have been specified by keyword.

	@param	paramName	the keyword of the desired parameter
	@param	outType		the resulting parameter value
	
	@blueworld
*/
	//osError lasso_findTagParam2( lasso_request_t token, const char * paramName, lasso_type_t * result );
	public native int findTagParam2( String paramName, LassoTypeRef outType );

/**
	Retrieves a parameter that was passed to the LJAPI tag call.

	The parameter must have been specified by keyword.

	@param	paramName	the keyword of the desired parameter
	@return				the resulting parameter value, or <code>null</code> if <code>paramName</code> keyword wasn't found
	
	@since v8.0
	
	@omnipilot
*/
	private LassoTypeRef findTagParam2( String paramName )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return ( this.findTagParam2( paramName, outType ) != NOERR) ? null : outType;
	}
	
/*********************************************************
	CHILDREN
**********************************************************/
/**
	Execute the contents of a container tag.
	
	Tags become containers by registering with the flag_typeContainerTag flag.
	The resulting data will be of type string.
	
	@param	outResult	the resulting data
	
	@blueworld
 */
	//osError lasso_childrenRun( lasso_request_t token, lasso_type_t * result );
	public native int childrenRun( LassoTypeRef outResult );

/**
	Execute the contents of a container tag.
	
	Tags become containers by registering with the {@link LassoDSModule#FLAG_CONTAINER FLAG_CONTAINER} flag.
	The resulting data will be of type string.
	
	@return		the resulting data
	
	@since v8.0
	
	@omnipilot
 */
	private LassoTypeRef childrenRun ( )
	{
		LassoTypeRef outType = new LassoTypeRef();
		return (this.childrenRun( outType ) != NOERR) ? null : outType;	
	}
	
// functions related to querying a tag's parameter and return value types

/**
	Provides access to tag parameters as specified by the <i>-returntype</i> keyword
	@since v7.0
	
	@blueworld
*/
//osError lasso_tagGetReturnType(lasso_request_t token, lasso_type_t tagType, auto_lasso_value_t * result);
public native int tagGetReturnType( LassoTypeRef inTagType, LassoValue outResult );

/**
	Provides access to tag parameters as specified by the <i>-returntype</i> keyword
	@since v8.0
	
	@omnipilot
*/
private String tagGetReturnType( LassoTypeRef tag )
{
	LassoValue outValue = new LassoValue();
	return (this.tagGetReturnType( tag, outValue ) != NOERR) ? null : outValue.name();	
}

/**
	Provides access to tag parameters as specified by the <i>-returntype</i> keywords
	@since v7.0
*/
//osError lasso_tagSetReturnType(lasso_request_t token, lasso_type_t tagType, const char * typeName);
public native int tagSetReturnType( LassoTypeRef inTagType, String typeName );
/**
	Provides access to tag parameters as specified by the <i>-required</i>/<i>-optional</i> keywords.
	@since v7.0
	
	@blueworld
*/
//osError lasso_tagGetExpectedParamCount(lasso_request_t token, lasso_type_t tagType, int * count);
public native int tagGetExpectedParamCount( LassoTypeRef inTagType, IntValue outCount );

/**
	Provides access to tag parameters as specified by the <i>-required</i>/<i>-optional</i> keywords.
	@since v8.0
	
	@omnipilot
*/
private int tagGetExpectedParamCount( LassoTypeRef tag )
{
	IntValue outCount = new IntValue();
	return (this.tagGetExpectedParamCount( tag, outCount ) != NOERR) ? -1 : outCount.intValue();	
}

/**
	Provides access to indexed tag parameter
	@since v7.0
*/
//osError lasso_tagGetExpectedParam(lasso_request_t token, lasso_type_t tagType, int index, auto_lasso_value_t * result, bool * required);
public native int tagGetExpectedParam( LassoTypeRef inTagType, int index, LassoValue outResult, IntValue outRequired );



/*********************************************************
	MORE CONVENIENCE
**********************************************************/




/*********************************************************
	MISC
**********************************************************/


// Private object storage

/**
	<code>typeSetCustomJavaObject</code> permits attachment of Java objects to custom types.
	Java object is retained until the <code>typeFreeCustomJavaObject</code> is called, or the 
	type is destroyed.<br>
	<p> One of the most common uses of this feature is to attach the
	{@link LassoTagModule LassoTagModule} instance to a custom LDML type created by that module. 
	In this case, Lasso will attempt to reuse the Java object and invoke member methods on the 
	attached module instead of creating new module instance for each corresponding call.
	As an added bonus, module methods will always be executed in the current "context". 
	So, instead of:
	<pre><code>
 LassoTypeRef type = new LassoTypeRef();
 lasso.getTagSelf(type);
 Object obj = lasso.typeGetCustomJavaObject(type);
 obj.counter++;</code><br>
one could simply write:<br><br><code>
 this.counter++;
 </code>
	</pre>
	Note: if the attached object class implements {@link java.lang.Cloneable Cloneable} 
	interface, an attempt will be made to invoke its <code>clone</code> method during assignment
	operation if <code>OnAssign</code> member is not implemented by the module.
 */
public native int typeSetCustomJavaObject( LassoTypeRef type, Object object );

/**
	Retrieves the Java object previously attached to a custom type.
	Returns <code>null</code> if no object has been associated with this custom type instance.
 */
public native Object typeGetCustomJavaObject( LassoTypeRef type );

/**
	Releases the Java object previously attached to a custom type.
	Must be called to free the Java object that is no longer needed, or 
	to detach a Java object before attaching a new one.
 */
public native int typeFreeCustomJavaObject( LassoTypeRef type );

// Assignment

/**
<code>typeAssign</code> performs an assignment of one type to another.
The result will be the same as if the following had been executed in LDML:
<p><code>#left_hand_side = #right_hand_side</code>
 */
//osError lasso_typeAssign( lasso_request_t token, lasso_type_t left_hand_side, lasso_type_t right_hand_side );
public native int typeAssign( LassoTypeRef left_hand_side, LassoTypeRef right_hand_side);

/**
Prevents the type from being destroyed once the LJAPI call returns.
Types that have been detached must eventually be destroyed using <code>typeFree</code>
or a memory leak will occur.
 */
// osError lasso_typeDetach( lasso_request_t token, lasso_type_t toDetach );
public native int typeDetach( LassoTypeRef toDetach );

/**
Creates and runs a new LJAPI call on the given Java module method. 
If there is already an active request on the current thread, the <code>methodName</code> will be run within that threads context. 
If there is no currently active request on the current thread, a new request will be created and run based off the global context.

@param	className	name of the java class implementing the method
@param	methodName	java method. Must have the same signature as {@link LassoTagModule#TAG_METHOD_PROTOTYPE TAG_METHOD_PROTOTYPE}
@param	tagAction	this parameter will be passed to the <code>methodName</code> and can be used to signal or pass information to the function
@param	unused	reserved for future use 
 */
//osError lasso_runRequest( lasso_tag_func func, tag_action_t action, int unused );
public static native int runRequest( String className, String methodName, int tagAction, int unused );


/*
	Get current site ID
	@since v8.0
 */
	//osError lasso_getSiteID( lasso_request_t token, osInt64 * outID );
	public native int getSiteID( );

/*
	Get current site name
	@since v8.0
 */
	//osError lasso_getSiteName(lasso_request_t token, auto_lasso_value_t * outName);
	public native String getSiteName( );
	


/*********************************************************
 LOGGING FUNCTIONS
*********************************************************/

/** 
Log a message. The message goes to the prefered destination for the message level.
Messages are limited to 2048 bytes in length.
Messages sent to the console are limited to 512 bytes in length.
Messages sent to the database are limited a little less than 2048 bytes since the 
total length of the sql statement used to insert the message is limited to 2048 bytes.

@param	msgLevel	one of the LOG_LEVEL constants
@param	message	message text
 */ 
public native static int log ( int msgLevel, String message );

//public native static LassoCall instanceOf( LassoCall lasso, boolean swap ); 

// main entry
private static void main ( String params[] )
{
	System.out.println ( version );
}

}
