package com.blueworld.lassopro;

/**
 * Base class for any datasource module.
 *
 * <code>LassoDSModules</code> are used to manipulate datasources.
 * <code>LassoDSModules</code> are looked up by the datasource names they claim to support.
 * They are instantiated once and used repeatedly by Lasso.
 *
 * <p>Every <code>LassoDSModule</code> must implement {@link LassoJavaModule#registerLassoModule registerLassoModule} 
 * method, and one or more methods with the same signature as {@link #DS_METHOD_PROTOTYPE DS_METHOD_PROTOTYPE}.<br><br>
 *
 * Lasso calls <code>registerLassoModule</code> once at startup to give this
 * module a chance to register its datasource(s). <code>LassoDSModule</code> must then 
 * call {@link #registerDSModule registerDSModule} as many times as there are datasources implemented by this module.
 */
public abstract class LassoDSModule implements LassoJavaModule
{
	/**
	 *	Datasource is being initialized
	 */
	public static final int ACTION_INIT		=	0;
	/**
	 *	Datasource is being terminated
	 */
	public static final int ACTION_TERM		=	1;
	
	/**
	 *	Does the specified datasource exist? 
	 *	Must return <code>ERR_NOERR</code> to indicate that it does.
	 *	Any other return value means that it doesn't exist.
	 */
	public static final int ACTION_EXISTS	=	2;
	
	/** Return the names of all the datasources */
	public static final int ACTION_DB_NAMES =	3;
	
	/** Return the names of the schemas in the specified datasource  */	
	public static final int ACTION_SCHEMA_NAMES	=	12;	// LP 6.0.2
	
	/** Return the names of the tables in the specified datasource  */
	public static final int ACTION_TABLE_NAMES	=	4;

	// Main datasource action types
		
	/** Perform search on a datasource using specified criteria */				
	public static final int ACTION_SEARCH	=	5;	
	/** Add new record to a database */
	public static final int ACTION_ADD		=	6;
	/** Update values in a database record */
	public static final int ACTION_UPDATE	=	7;
	/** Delete database record */
	public static final int ACTION_DELETE	=	8;
	
	/** Return the column info */			
	public static final int ACTION_INFO		=	9;
	
	/** Only needs to be supported by SQL-based databases */
	public static final int ACTION_SQL		=	10;

	/** Return random database record */	
	public static final int ACTION_RANDOM	=	11;	

	/** Return random database record */	
	public static final int ACTION_CLOSE	=	13;	 // LCAPI 7.0

	/** 
		Sent to the datasource when a <code>-database</code> and <code>-table</code> are specified, but no action (or a <code>-nothing</code> action).
		The database can, perhaps, set or reset its connection to the database. Or, it can do nothing.		
	*/									
	public static final int ACTION_TICKLE	=	14;	 // LCAPI 7.0
		
	// Constants defining sort order.
	 
	/** Sort records in ascending order (<code>ASC</code>) */	 
	public static final int SORT_ASCENDING = 0;
	/** Sort records in descending order (<code>DESC</code>) */	
	public static final int SORT_DESCENDING = 1;
	/** Use custom order for ordering this column's values*/	
	public static final int SORT_CUSTOM = 4;
	
	/** This column's values can be updated */
	public static final int PROTECTION_NONE 	 = 0x6E6F6E65; // 'none'
	/** This column's value is read-only */
	public static final int PROTECTION_READ_ONLY = 0x6E6D6F64; // 'nmod'
	
	
	private int nativeRef;
	{ nativeRef = 0; }
	
	protected LassoDSModule()
	{
		super();
	}
	/**
		Registers this datasource with Lasso Professional 6.
	
		@param moduleName the name of your datasource connector, as returned by <code>[Lasso_DatasourceModuleName]</code> LDML tag
		@param methodName the name of your module (class) main entry method. <code>methodName</code> method should have the same signature as <code>DS_METHOD_PROTOTYPE</code>
		@param flags unused
		@param longName unused
		@param description short description of your datasource connector. Can be null.
	*/	
	protected native void registerDSModule(String moduleName, String methodName, int flags, String longName, String description);

	/**
	* A prototype for all datasource methods registered by <code>registerDSModule</code>.
	* Since methods are being registered by name, they must exactly match
	* the value passed in a <code>methodName</code> parameter of <code>registerDSModule</code> call.
	* 
	* @param lasso an object to be used for invoking various <code>LassoCall</code> methods
	* @param action one of the database <code>ACTION_</code> constants
	* @param data datasource, host, or database name when <code>action</code> is one of the following:
	* <code>ACTION_EXISTS</code>, <code>ACTION_DB_NAMES</code>, <code>ACTION_SCHEMA_NAMES</code> or <code>ACTION_TABLE_NAMES</code>. 
	* If <code>action</code> is <code>ACTION_SQL</code>, the <code>data</code> member of <code>LassoValue</code> object
	* contains the full text of the SQL statement passed to the inline.
	*/
	public int DS_METHOD_PROTOTYPE(LassoCall lasso, int action, LassoValue data ) { return 0; }
}
