package com.blueworld.lassopro;

import com.blueworld.lassopro.LassoJavaModule;

/**
 * Base class for any tag module.
 * Most tag modules output data onto the web page, though some tags may
 * perform other actions based on the parameters passed to them.<br><br>
 *
 * Every <code>LassoTagModule</code> must implement {@link LassoJavaModule#registerLassoModule registerLassoModule} 
 * method, and one or more methods with the same signature as {@link #TAG_METHOD_PROTOTYPE TAG_METHOD_PROTOTYPE}.<br><br>
 *
 * Lasso calls <code>registerLassoModule</code> once at startup to give this
 * module a chance to register its tags. <code>LassoTagModule</code> must then 
 * call <code>registerTagModule</code> as many times as there are tags implemented by this module.
 */
public abstract class LassoTagModule implements LassoJavaModule
{
	/**
	 * Type initializer tags can have their own members.
	 */
	public static final int FLAG_INITIALIZER 	=	1;
	/**
	 * Regular substitution tag.
	 */
	public static final int FLAG_SUBSTITUTION	=	2;
	/**
	 * Async tags run asynchronously in their own thread.
	 */
	public static final int FLAG_ASYNC			=	4;	
	/**
	 * Container tags have opening and closing. This flag will cause Lasso Professional to raise an error
	 * if the closing [/xxx] tag can't be found.
	 */
	public static final int FLAG_CONTAINER		=	8;
	/**
	 * CAUTION! Specifying this flag will effectively bypass any security checks performed by Lasso Service at the run time.
	 * Use at your own risk!
	 */
	public static final int FLAG_SKIP_SECURITY_CHECK 	=	0x00000040;
	
	private int nativeRef;
	{ nativeRef = 0; }

	protected LassoTagModule()
	{
		super();
	}
		
 /**
  * Use this method to register substitution tags implemented by your module.
  * You should call <code>registerTagModule</code> as many times as there are tags implemented in your module.
  *
  * @param moduleName name of the module as returned by <code>[Lasso_TagModuleName]</code> LDML tag
  * @param tagName name of the custom LDML tag implemented by this module
  * @param methodName Lasso will call this method every time it finds <code>tagName</code> tag in a <code>xxx.lasso</code> format file
  * @param flags one or more logically or'ed <code>FLAG</code> constants
  * @param description optional tag info, such as brief description of this tag usage
  */
	protected native void registerTagModule(String moduleName, String tagName, String methodName, int flags, String description);
		
 /**
  * A prototype for all tag methods registered by <code>registerTagModule</code>.
  * Since methods are being registered by name, they must exactly match
  * the values passed in a <code>tagName</code> parameter of <code>registerTagModule</code> call.
  *
  * @param action reserved for future use;
  */
	public int TAG_METHOD_PROTOTYPE( LassoCall lasso, int action ) { return 0; }
}
