<?LassoScript

	define_tag: 'TranslateBreaksToCRLF',
			-required = 'data',
			-namespace = 'email_';
		return: @(string_replaceregexp:
			-find = '\r\n|\n|\r',
			-replace = '\r\n',
			#data);
	/define_tag;


	// [Email_FindEmails]
	//
	// First parameter is a string containing one or more email address.
	//
	// This tag returns an array of all the email addresses contained within a string.
	//

	define_tag: 'FindEmails',
		-Priority='Replace',
		-namespace = 'email_';
		Local: 'emailArray' = (String_Replace: params->(get: 1), -find=';', -replace=',');
		Local: 'emailArray' = (String_FindRegExp: #emailArray, -find='[^ @\r\n,]*@[^ @\r\n,]*');
		Local: 'outArray' = (array);
		Loop: #emailArray->size;
			If: #emailArray->(get: loop_Count)->(get: 1,1) == '<';
				#outArray->(insert: (String_FindRegExp: #emailArray->(get: Loop_Count), -Search='<([^@]*@[^@]*)>')->get:2);
			else;
				If: #emailArray->(get: loop_Count)->(get: 1,1) != '"';
					#outArray->(insert: #emailArray->(get: Loop_Count));
				/If;
			/If;
		/Loop;
		return: #outArray;
	/define_tag;

	// [Email_Compose] Type
	//
	// This type functions as a component of [Email_Send] and allows for emails
	// with attachments, HTML and text parts, and custom encodings to be created.
	//
	// [Email_Compose]
	//
	// The following four member tags allow different body parts and attachments to be added.
	//
	// [Email_Compose->AddTextPart]
	// [Email_Compose->AddHTMLPart]
	// [Email_Compose->AddAttachment]
	// [Email_Compose->AddPart]
	//
	// The following three member tags are used by [Email_Send]
	//
	// [Email_Compose->Recipients] - Returns an array of recipients of the message.  Includes to, cc, and bcc addresses.
	// [Email_Compose->From] - Returns the from address for the message.
	// [Email_Compose->Data] - Returns the MIME text of the message including all parts and attachments.
	// [Email_Compose->Summary] - Returns a summary of the essential headers of the message for display.
	//


	define_type: 'compose',
		-namespace = 'email_';

		local: '_msg';
		local: '_hdrs';
		local: '_recipients';
		local: '_from';
		local: '_contenttype';
		local: '_transferencoding';
		local: '_contentdisposition';
		local: '_charset';
		local: '_boundary';
		local: '_parts';
		local: '_mode';

		define_tag: 'onCreate',
				-optional='date',
				-optional='from',
				-optional='to',
				-optional='cc',
				-optional='bcc',
				-optional='subject',
				-optional='sender',
				-optional='replyto',
				-optional='body',
				-optional='html',
				-optional='contenttype',
				-optional='characterset',
				-optional='transferencoding',
				-optional='contentdisposition',
				-optional='extramimeheaders',
				-optional='attachments';

			fail_if: !(local_defined: 'from'), -1, '[Email_Compose] A from address is required';
			fail_if: !((local_defined: 'to') || (local_defined: 'cc') || (local_defined: 'bcc')), -1, '[Email_Compose] At least one recipient (to,cc,bcc) address is required';

			//
			// Recipients
			//
			self->'_recipients' = (array);
			if: (local_defined: 'to');
				self->'_recipients'->(merge: (Email_FindEmails: #to));
			/if;
			if: (local_defined: 'cc') && (local: 'cc')->size;
				self->'_recipients'->(merge: (Email_FindEmails: #cc));
			/if;
			if: (local_defined: 'bcc') && (local: 'bcc')->size;
				self->'_recipients'->(merge: (Email_FindEmails: #bcc));
			/if;

			//
			// From
			//
			self->'_from' = '';
			if: (local_defined: 'from');
				self->'_from' = (Email_FindEmails: #from)->(first);
			/if;

			//
			// Headers
			//
			self->'_hdrs' = (array);
			if: (local_defined: 'extramimeheaders') && #extramimeheaders->(isa: 'array');
				self->'_hdrs'->(merge: #extramimeheaders);
			/if;

			if: (self->'_hdrs' !>> 'MIME-Version');
				self->'_hdrs'->(insert: 'MIME-Version' = '1.0');
			/if;
			if: (local_defined: 'characterset');
				self->'_charset' = #characterset;
			else;
				self->'_charset' = 'utf-8';
			/if;
			if: (local_defined: 'contenttype');
				self->'_hdrs'->(removeall: 'Content-Type');
				if: #contenttype >> 'charset';
					self->'_hdrs'->(insert: 'Content-Type' = #contenttype);
					self->'_contenttype' = #contenttype;
					self->'_charset' = '';
				else;
					self->'_hdrs'->(insert: 'Content-Type' = #contenttype + '; charset=' + (self->'_charset'));
					self->'_contenttype' = #contenttype + '; charset=' + (self->'_charset');
				/if;
			else: (self->'_hdrs' !>> 'Content-Type');
				self->'_hdrs'->(insert: 'Content-Type' = 'text/plain; charset=' + (self->'_charset') '');
				self->'_contenttype' = 'text/plain; charset=' + (self->'_charset') '';
			else;
				protect;
					self->'_contenttype' = self->'_hdrs'->(find: 'Content-Type')->first->second;
				/protect;
			/if;
			if: (self->'_charset' == '');
				protect;
					self->'_charset' = (string_findregexp: self->'_hdrs'->(find: 'Content-Type')->first->second, -find='(?<=charset=)[0-9a-zA-Z_\'()+_,\\-./:=?]+(?=)')->(get:1);
				/protect;
			/if;
			if: (self->'_charset' == '');
				protect;
					self->'_charset' = (string_findregexp: self->'_hdrs'->(find: 'Content-Type')->first->second, -find='(?<=charset=)[0-9a-zA-Z_\'()+_,\\-./:=?]+')->(get:1);
				/protect;
			/if;

			if: (local_defined: 'transferencoding');
				self->'_hdrs'->(removeall: 'Content-Transfer-Encoding');
				self->'_hdrs'->(insert: 'Content-Transfer-Encoding' = #transferencoding);
				self->'_transferencoding' = #transferencoding;
			else: (self->'_hdrs' !>> 'Content-Transfer-Encoding');
				self->'_hdrs'->(insert: 'Content-Transfer-Encoding' = 'quoted-printable');
				self->'_transferencoding' = 'quoted-printable';
			else;
				protect;
					self->'_transferencoding' = self->'_hdrs'->(find: 'Content-Transfer-Encoding')->first->second;
				/protect;
			/if;

			if: (self->'_hdrs' !>> 'X-Mailer');
				self->'_hdrs'->(insert: 'X-Mailer' = 'Lasso Professional ' + (string_replaceregexp: -find = '([\\.\\d]+).*', -replace = '\\1', (lasso_version: -lassoversion)));
			/if;

			if: (self->'_hdrs' !>> 'Message-ID');
				self->'_hdrs'->(insert: 'Message-ID' = '<' + lasso_uniqueID + '@' + {
						if: (global_defined: '__smtp_ip__');
							return: $__smtp_ip__;
						else: (admin_getpref: (map: 'lasso_smtp_lassoservice_ip' = ''))->(find: 'lasso_smtp_lassoservice_ip')->size;
							global: '__smtp_ip__' = (admin_getpref: (map: 'lasso_smtp_lassoservice_ip' = ''))->(find: 'lasso_smtp_lassoservice_ip');
						else;
							global: '__smtp_ip__' = '127.0.0.1';
						/if;

						return: $__smtp_ip__;
					}->eval + '>');
			/if;

			if: (local_defined: 'date') && #date->(isa: 'date');
				local: 'timezone' = #date->(format: '%z');
				if: !(#timezone->(beginswith: '-')) && !(#timezone->(beginswith:'+'));
					#timezone = '+' + #timezone;
				/if;
				self->'_hdrs'->(insert: 'Date' = #date->(format: '%a, %-d %b %Y %T ') + #timezone );
			else;
				local: 'timezone' = date->(format: '%z');
				if: !(#timezone->(beginswith: '-')) && !(#timezone->(beginswith:'+'));
					#timezone = '+' + #timezone;
				/if;
				self->'_hdrs'->(insert: 'Date' = date->(format: '%a, %-d %b %Y %T ') + #timezone );
			/if;

			if: (local_defined: 'to');
				self->'_hdrs'->(insert: 'To' = #to);
			/if;
			if: (local_defined: 'cc') && (local: 'cc')->size;
				self->'_hdrs'->(insert: 'Cc' = #cc);
			/if;
			if: (local_defined: 'from');
				self->'_hdrs'->(insert: 'From' = #from);
			/if;
			if: (local_defined: 'subject');
				self->'_hdrs'->(insert: 'Subject' = #subject);
			/if;

			if: (local_defined: 'sender');
				self->'_hdrs'->(insert: 'Sender' = #sender);
			/if;

			if: (local_defined: 'replyto');
				self->'_hdrs'->(insert: 'Reply-To' = #replyto);
			/if;

			self->'_parts' = (array);
			self->'_mode' = '';

			if: (local_defined: 'body');
				self->(addtextpart: -data=#body);
			/if;

			if: (local_defined: 'html');
				self->(addhtmlpart: -data=#html);
			/if;

			if: (local_defined: 'attachments');
				iterate: (if_false: #attachments->(isa: 'array'), (array: #attachments), #attachments), (local: 'attachment');
					self->(addattachment: -path=#attachment);
				/iterate;
			/if;
		/define_tag;

		define_tag: 'addtextpart',
			-optional='data',
			-optional='path';

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else: (local_defined: 'path');
				local: '_raw' = (include: #path);
			else;
				fail: -1, '[Email_Compose] The text part could not be found.';
			/if;

			#_raw = (Email_TranslateBreaksToCRLF: #_raw);

			local: 'part' = (map:
				'_body' = #_raw,
				'_hdrs'=(array:
					'Content-Type' = (self->'_contenttype'),
					'Content-Transfer-Encoding' = (self->'_transferencoding'),
				),
				'_contenttype' = (self->'_contenttype'),
				'_transferencoding' = (self->'_transferencoding'),
				'_charset' = (self->'_charset'),
			);
			self->'_parts'->(insert: #part);
			self->'_mode' += 'text ';
		/define_tag;

		define_tag: 'addhtmlpart',
			-optional='data',
			-optional='path';

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else: (local_defined: 'path');
				local: '_raw' = (include: #path);
			else;
				fail: -1, '[Email_Compose] The HTML part could not be found.';
			/if;

			#_raw = (Email_TranslateBreaksToCRLF: #_raw);

			local: 'part' = (map:
				'_body' = #_raw,
				'_hdrs'=(array:
					'Content-Type' = 'text/html; charset=' + (self->'_charset'),
					'Content-Transfer-Encoding' = (self->'_transferencoding'),
				),
				'_contenttype' = 'text/html; charset=' + (self->'_charset'),
				'_transferencoding' = (self->'_transferencoding'),
				'_charset' = (self->'_charset'),
			);
			self->'_parts'->(insert: #part);
			self->'_mode' += 'html ';
		/define_tag;

		define_tag: 'addattachment',
			-optional='data',
			-optional='name',
			-optional='path';

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else: (local_defined: 'path');
				local: '_raw' = (include_raw: #path);
			else;
				fail: -1, '[Email_Compose] An attachment could not be found.';
			/if;

			if: (#_raw == '');
				fail: -1, '[Email_Compose] The attachment was empty.';
			/if;

			if: (local_defined: 'name');
				local: '_name' = #name;
			else: (local_defined: 'path');
				local: '_name' = #path->(split: '/')->last->(split:'\\')->last;
			else;
				fail: -1, '[Email_Compose] An attachment name must be provided.';
			/if;

			local: '_base64' = (encode_base64: #_raw);
			local: '_array' = (string_findregexp: #_base64, -find='\\S{0,76}');

			local: 'part' = (map:
				'_body' = #_array->(join: '\r\n'),
				'_hdrs'=(array:
					'Content-Type' = 'application/octet-stream; name="' + #_name + '"',
					'Content-Transfer-Encoding' = 'base64',
					'Content-Disposition' = 'attachment; filename="' + #_name + '"',
					'Content-ID' = '<' + (encode_xml: #_name) + '>',
				),
				'_contenttype' = 'application/octet-stream; name="' + #_name + '"',
				'_transferencoding' = 'base64',
			);
			self->'_parts'->(insert: #part);
			self->'_mode' += 'attachment ';
		/define_tag;

		define_tag: 'addpart',
			-optional='data';

			if: (local_defined: 'data');
				local: '_raw' = #data;
			else;
				fail: -1, '[Email_Compose] A part could not be found.';
			/if;

			self->'_parts'->(insert: #_raw);
			self->'_mode' += 'part ';
		/define_tag;

		define_tag: 'data';
			local: 'output' = (array);

			self->'_boundary' = '====_-' + (lasso_uniqueid) + '-_====';

			local: 'multi_part' = (self->'_parts'->size > 1);

			if: #multi_part;
				self->'_hdrs'->(removeall: 'Content-Type');
				self->'_hdrs'->(insert: 'Content-Type' = 'multipart/alternative; boundary="' + self->'_boundary' + '"');
				self->'_contenttype' = 'multipart/alternative';

				self->'_hdrs'->(removeall: 'Content-Transfer-Encoding');

			else: (self->'_parts'->size > 0);
				self->'_hdrs'->(removeall: 'Content-Type');
				self->'_hdrs'->(merge: self->'_parts'->first->(find: '_hdrs')->(find: 'content-type'));

				self->'_hdrs'->(removeall: 'Content-Transfer-Encoding');
				self->'_hdrs'->(merge: self->'_parts'->first->(find: '_hdrs')->(find: 'content-transfer-encoding'));
			/if;

			iterate: self->'_hdrs', (local: 'hdr');
				// Only encode certain headers that we recognize
				if: ((array: 'subject','comment','to','cc','bcc','from','reply-to','sender') >> #hdr->first);
					#output->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'_charset'));
				else;
					#output->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset='none'));
				/if;
			/iterate;

			#output->(insert: '');

			if: (self->'_parts'->size > 0);

				if: #multi_part;
					#output->(insert: '--' + self->'_boundary');
				/if;

				iterate: self->'_parts', (local: 'part');
					if: #multi_part;
						iterate: #part->(find: '_hdrs'), (local: 'hdr');
							#output->(insert: (encode_qheader: -name=#hdr->first, -value=#hdr->second, -charset=self->'_charset'));
						/iterate;

						#output->(insert: '');
					/if;

					if: (#part->(find: '_transferencoding') >> 'quoted-printable');
						#output->(insert: (encode_quotedprintable: #part->(find: '_body'), -charset=#part->(find: '_charset')));
					else: (string_validcharset: #part->(find: '_charset'));
						#output->(insert: (bytes: #part->(find: '_body'), #part->(find: '_charset')));
					else;
						#output->(insert: #part->(find: '_body'));
					/if;

					if: #multi_part;
						#output->(insert: '');
						if: loop_count == self->'_parts'->size;
							#output->(insert: '--' + self->'_boundary' + '--');
						else;
							#output->(insert: '--' + self->'_boundary');
						/if;
					/if;

				/iterate;

			/if;

			local: 'return' = bytes;
			local: 'valid' = (string_validcharset: self->'_charset');
			iterate: #output, (local: 'temp');
				if: #temp->(isa: 'bytes');
					#return += #temp;
				else: (#valid == true);
					#return += (bytes: (string: #temp), self->'_charset');
				else;
					#return += (bytes: (string: #temp));
				/if;
				#return += '\r\n';
			/iterate;

			return: @#return;

		/define_tag;

		define_tag: 'recipients';
			return: self->'_recipients';
		/define_tag;

		define_tag: 'from';
			return: self->'_from';
		/define_tag;

		define_tag: 'onConvert';
			return: self->data;
		/define_tag;

	/define_type;

	// [Email_Send]
	//
	// This tag is a drop-in replacement for [Email_Send].  It uses [Email_Compose] on the back-end in order
	// to create the message to be sent.
	//

	Define_Tag: 'Send',
		-Priority='replace',
		-namespace = 'email_',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout',
		-Optional='priority',
		-Optional='to',
		-Optional='cc',
		-Optional='bcc',
		-Optional='from',
		-Optional='replyto',
		-Optional='sender',
		-Optional='subject',
		-Optional='body',
		-Optional='html',
		-Optional='transferencoding',
		-Optional='contenttype',
		-Optional='characterset',
		-Optional='attachments',
		-Optional='extramimeheaders',
		-Optional='simpleform';

		fail_if: ((local_defined: 'from') && (!#from->size)) || (!(local_defined: 'from')), -1, '[Email_Send] Requires a -From address.';
		fail_if: !(local_defined: 'subject'), -1, '[Email_Send] Requires a -Subject.';
		fail_if: !(((local_defined: 'to') && (#to->size > 0)) || ((local_defined: 'cc') && (#cc->size > 0)) || ( (local_defined: 'bcc') && (#bcc->size > 0))), -1, '[Email_Send] Requires a recipient.';

		local(
			'_params' = params,
			'_host' = string(local('host')),
			'_username' = string(local('username')),
			'_password' = string(local('password'))
		);

		if((!#_username->size) && (!#_password->size) && (#_host >> '||') && (#_host->split('||')->size == 3));
			#_username = #_host->split('||')->get(1);
			#_password = #_host->split('||')->get(2);
			#_host = #_host->split('||')->get(3);
		else(#_host >> '||');
			#_host = #_host->split('||')->last;
		/if;

		if: (local_defined: 'simpleform');
			#_params->(removeall: '-simpleform');
			#_params->(removeall: '-body');

			if: !(local_defined: 'body');
				local: '_body' = string;
			else;
				local: '_body' = #body + '\n\n';
			/if;

			iterate: action_params, (local: 'i');
				if: !#i->first->(beginswith: '-');
					#_body += #i->first ': ' #i->second '\n';
				/if;
			/iterate;

			#_body += '\nForm was submitted by: ' client_ip '\n';
			#_body += 'Form was submitted on: ' date->(format: '%-D %-r %Z') '\n';
		
			#_params->(removeall: '-body');
			#_params->(insert: -body = @#_body);
		/if;

		local: 'data' = @(\email_compose->(run: -params=#_params, -name='email_compose'));
		local: 'mime_from' = #data->from;
		local: 'mime_recipients' = #data->recipients;
		local: 'mime_text' = #data->data;

		fail_if: (#mime_text == ''), -1, '[Email_Send] The MIME encoding for the email message could not be generated.';
		fail_if: (#mime_recipients == ''), -1, '[Email_Send] Requires one or more valid email addresses for recipients.';
		fail_if: (#mime_from == ''), -1, '[Email_Send] Requires a -From parameter with the sender\'s email address.';

		Email_Queue:
			-data=#mime_text,
			-from=#mime_from,
			-recipients=#mime_recipients,
			-priority=(local: 'priority'),
			-host = #_host,
			-port = (local: 'port'),
			-timeout = (local: 'timeout'),
			-username = #_username,
			-password = #_password;

	/Define_Tag;


	//
	// [Email_Queue]
	//

	Define_Tag: 'Queue', -Priority='replace',
		-Privileged,
		-namespace = 'email_',
		-Required='data',
		-Required='recipients',
		-Required='from',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout',
		-Optional='priority';

		fail_if: (#data == ''), -1, '[Email_Queue] Requires a -Data parameter containing a MIME encoded email message.';
		fail_if: (#recipients == ''), -1, '[Email_Queue] Requires a -Recipients parameter with one or more email addresses.';
		fail_if: (#from == ''), -1, '[Email2_Queue] Requires a -From parameter with the sender\'s email address.';

		if: ((global: '__SMTP_Defaults__') == True) || (local_defined: 'host');
			local: 'smtp_host' = (if_null: (local: 'host'), '');
			local: 'smtp_port' = (if_null: (local: 'port'), '');
			local: 'smtp_username' = (if_null: (local: 'username'), '');
			local: 'smtp_password' = (if_null: (local: 'password'), '');
			local: 'smtp_timeout' = (if_null: (local: 'timeout'), '');
		else;
			fail: -1, '[Email_Queue] Requires a -Host parameter (or defaults established in Lasso Admin).';
		/if;

		local: 'queue_priority' = 2;
		if: (local_defined: 'priority') && (#priority->(isa: 'string'));
			if: #priority >> 'high';
				local: 'queue_priority' = 1;
			else; #priority >> 'low';
				local: 'queue_priority' = 3;
			/if;
		/if;

		inline:
			-log=$__startup_loglevel__,
			-database='site',
			-table='smtp_queue',
			-sql='INSERT INTO smtp_queue \
				(mime_text,mime_recipients,mime_from,smtp_host,smtp_port,smtp_timeout,smtp_username,smtp_password,queue_date,queue_status,queue_error,queue_number,queue_priority,create_date) \
				VALUES (\ 
				X"' + (encode_hex: #data) + '", \
				"' + (encode_sql92: (if_false: #recipients->(isa: 'array'), (array: #recipients), #recipients)->(join: ',')) + '", \
				"' + (encode_sql92: #from) + '", \
				"' + (encode_sql92: #smtp_host) + '", \
				"' + (encode_sql92: #smtp_port) + '", \
				"' + (encode_sql92: #smtp_timeout) + '", \
				"' + (encode_sql92: #smtp_username) + '", \
				"' + (encode_sql92: #smtp_password) + '", \
				"' + (encode_sql92: date->(format: '%Q %T')) + '", \
				"queued", \
				"", \
				0, \
				3, \
				"' + (encode_sql92: date->(format: '%Q %T')) + '")';
			if: error_code != 0;
				log_critical: 'Queue Error: ' error_msg;
			/if;
		/inline;

		if: ((Global: '__SMTP_Status__') !>> 'Abort') && ((Global: '__SMTP_Status__') !>> 'Pause');
			Global: '__SMTP_Status__' = 'Active';
		/if;

	/Define_Tag;

	Define_Tag: 'Immediate', -Priority='replace',
		-Privileged,
		-namespace = 'email_',
		-Required='data',
		-Required='recipients',
		-Required='from',
		-Optional='host',
		-Optional='username',
		-Optional='password',
		-Optional='port',
		-Optional='timeout';

		fail_if: (#data == ''), -1, '[Email_Immediate] Requires a -Data parameter containing a MIME encoded email message.';
		fail_if: (#recipients == ''), -1, '[Email_Immediate] Requires a -Recipients parameter with one or more email addresses.';
		fail_if: (#from == ''), -1, '[Email_Immediate] Requires a -From parameter with the sender\'s email address.';

		Local: 'prefs' = (map: 
				'lasso_smtp_host' = '',
				'lasso_smtp_port' = 25,
				'lasso_smtp_username' = '',
				'lasso_smtp_password' = '',
				'lasso_smtp_timeout' = 15,
			);					
		Local: 'prefs' = (admin_getpref: #prefs);

		local: 'smtp_host' = (if_null: (local: 'host'), #prefs-(find: 'lasso_smtp_host'));
		local: 'smtp_port' = (if_null: (local: 'port'), #prefs-(find: 'lasso_smtp_port'));
		local: 'smtp_username' = (if_null: (local: 'username'), #prefs-(find: 'lasso_smtp_username'));
		local: 'smtp_password' = (if_null: (local: 'password'), #prefs-(find: 'lasso_smtp_password'));
		local: 'smtp_timeout' = (if_null: (local: 'timeout'), #prefs-(find: 'lasso_smtp_timeout'));

		fail_if: (#smtp_host == ''), -1, '[Email_Immediate] Requires a -Host parameter.';

		protect;
			local: 'smtp' = (email_smtp);
			handle_error;
				fail: -1, '[Email_Immediate] Failed to initialize SMTP connector.';
			/handle_error;
		/protect;
		protect;
			if: (#smtp_username != '') && (#smtp_password != '');
				#smtp->(open: -host=#smtp_host, -port=(integer: #smtp_port), -username=#smtp_username, -password=#smtp_password, -timeout=#smtp_timeout);
			else;
				#smtp->(open: -host=#smtp_host, -port=(integer: #smtp_port), -timeout=#smtp_timeout);
			/if;
			handle_error;
				fail: -1, '[Email_Immediate] Failed to connect to host "' + #smtp_host + '".';
			/handle_error;
		/protect;

		protect;
			#smtp->(send: -from=#from, -recipients=#recipients, -message=#data + '\r\n');
			handle_error;
				fail: -1, '[Email_Immediate] Error sending message';
			/handle_error;
		/protect;

		protect;
			#smtp->close;
		/protect;

	/Define_Tag;

?>
